package org.hisp.dhis.mobile.connection;

/*
 * Copyright (c) 2004-2010, University of Oslo All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution. * Neither the name of the HISP project nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;
import javax.microedition.io.ConnectionNotFoundException;
import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import javax.microedition.io.HttpsConnection;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.List;
import javax.microedition.rms.RecordStoreException;
import org.hisp.dhis.mobile.db.ModelRecordStore;
import org.hisp.dhis.mobile.db.OrgUnitStore;
import org.hisp.dhis.mobile.model.ActivityValue;
import org.hisp.dhis.mobile.model.DataSet;
import org.hisp.dhis.mobile.model.DataSetList;
import org.hisp.dhis.mobile.model.DataSetValue;
import org.hisp.dhis.mobile.model.DataStreamSerializable;
import org.hisp.dhis.mobile.model.MobileWrapper;
import org.hisp.dhis.mobile.model.Model;
import org.hisp.dhis.mobile.model.ModelList;
import org.hisp.dhis.mobile.model.OrgUnit;
import org.hisp.dhis.mobile.model.Program;
import org.hisp.dhis.mobile.ui.Alerts;
import org.hisp.dhis.mobile.ui.DHISMIDlet;
import org.hisp.dhis.mobile.ui.Text;
import org.hisp.dhis.mobile.util.RefreshDataSetListConfirmListener;
import org.hisp.dhis.mobile.util.SerializationUtil;

import com.jcraft.jzlib.ZInputStream;

public class ConnectionManager
    extends Thread
{

    private static final String MEDIATYPE_MOBILE_SERIALIZED = "application/vnd.org.dhis2.mobile+serialized";

    // Tasks
    private static final short AUTHENTICATE = 0;

    private static final short UPLOAD_DATASET_VALUES = 1;

    private static final short UPLOAD_ACTIVITY_VALUES = 2;

    private static final short DOWNLOAD_ALL = 3;

    private static final short UPDATE_ACTIVITYPLAN = 4;

    private static final short UPDATE_DATASET = 5;

    // Class variables

    private static DHISMIDlet dhisMIDlet;

    private static String url;

    private static String userName;

    private static String password;

    private static String ua;

    private static String locale;

    private static OrgUnit orgUnit;

    // Instance variables

    private short task;

    Vector abstractModelListVector = new Vector();

    private DataSetValue dataSetValue;

    private ActivityValue activityValue;

    private MobileWrapper mobileWrapper;

    private DataSetList dataSetList;

    private StringBuffer updateDataSetResult;

    public static void init( DHISMIDlet dhisMIDlet, String url, String userName, String password, String locale,
        OrgUnit orgUnit )
    {
        ConnectionManager.dhisMIDlet = dhisMIDlet;
        ConnectionManager.url = url;
        ConnectionManager.userName = userName;
        ConnectionManager.password = password;
        ConnectionManager.locale = locale;
        ConnectionManager.orgUnit = orgUnit;

        ua = "Profile/" + System.getProperty( "microedition.profiles" ) + " Configuration/"
            + System.getProperty( "microedition.configuration" );
    }

    private ConnectionManager()
    {
    }

    public static void login()
    {
        ConnectionManager connectionManager = new ConnectionManager();
        connectionManager.task = AUTHENTICATE;
        connectionManager.start();
    }

    public static void downloadAllResources()
    {
        ConnectionManager connectionManager = new ConnectionManager();
        connectionManager.task = DOWNLOAD_ALL;
        connectionManager.start();
    }

    public static void updateActivityPlan()
    {
        ConnectionManager connectionManager = new ConnectionManager();
        connectionManager.task = UPDATE_ACTIVITYPLAN;
        connectionManager.start();
    }

    public static void uploadFacilityReport( DataSetValue dataSetValue )
    {
        ConnectionManager connectionManager = new ConnectionManager();
        connectionManager.task = UPLOAD_DATASET_VALUES;
        connectionManager.dataSetValue = dataSetValue;
        connectionManager.start();
    }

    public static void uploadActivityReport( ActivityValue activityValue )
    {
        ConnectionManager connectionManager = new ConnectionManager();
        connectionManager.task = UPLOAD_ACTIVITY_VALUES;
        connectionManager.activityValue = activityValue;
        connectionManager.start();
    }

    public static void updatePrograms()
    {
        ConnectionManager connectionManager = new ConnectionManager();
        connectionManager.task = UPDATE_ACTIVITYPLAN;
        connectionManager.start();
    }

    public static void updateDataSets( Vector currentDataSets )
    {
        ConnectionManager connectionManager = new ConnectionManager();
        connectionManager.task = UPDATE_DATASET;
        for ( int i = 0; i < currentDataSets.size(); i++ )
        {
            ((DataSet) currentDataSets.elementAt( i )).setSections( null );
        }
        connectionManager.dataSetList = new DataSetList();
        connectionManager.dataSetList.setCurrentDataSets( currentDataSets );
        connectionManager.start();
    }

    public void run()
    {
        if ( task == ConnectionManager.UPLOAD_DATASET_VALUES )
        {
            uploadDataSetReport();
        }
        else if ( task == ConnectionManager.UPLOAD_ACTIVITY_VALUES )
        {
            uploadActivityReport();
        }
        else if ( task == ConnectionManager.AUTHENTICATE )
        {
            try
            {
                authenticate();
                // If OrgUnit is null, it means that there are more than one
                // OrgUnit. Do nothing, just wait for user to choose.
                // Otherwise, set OrgUnit and start download everything
                if ( orgUnit != null )
                {
                    OrgUnitStore.setOrgUnit( orgUnit );
                    dhisMIDlet.setOrgUnit( orgUnit );

                    dhisMIDlet.getPinForm().append( Text.CREATE_PIN_SUGGESTION() );
                    dhisMIDlet.switchDisplayable( null, dhisMIDlet.getPinForm() );

                    // download all after login (separated thread)
                    ConnectionManager.downloadAllResources();
                    dhisMIDlet.setDownloading( true );
                }
            }
            catch ( ConnectionNotFoundException e )
            {
                dhisMIDlet.switchDisplayable( Alerts.getErrorAlert( Text.ERROR(), Text.SERVER_NOT_FOUND() ),
                    dhisMIDlet.getLoginForm() );
                e.printStackTrace();
            }
            catch ( Exception e )
            {
                dhisMIDlet.switchDisplayable( Alerts.getErrorAlert( Text.ERROR(), e.getMessage() ),
                    dhisMIDlet.getLoginForm() );
                e.printStackTrace();
            }
        }
        else if ( task == ConnectionManager.DOWNLOAD_ALL )
        {
            try
            {
                downloadAll();
            }
            catch ( IOException e )
            {
                e.printStackTrace();
            }
            try
            {
                dhisMIDlet.saveActivityPlan( mobileWrapper.getActivityPlan() );
                Vector progs = mobileWrapper.getPrograms();
                Vector dataSets = mobileWrapper.getDataSets();
                dhisMIDlet.setServerCurrentDate( mobileWrapper.getServerCurrentDate() );
                dhisMIDlet.saveFacilityForms( dataSets );
                for ( int i = 0; i < progs.size(); i++ )
                {
                    dhisMIDlet.saveProgram( (Program) progs.elementAt( i ) );
                }

                if ( progs.size() <= 0 && dataSets.size() > 0 )
                {
                    dhisMIDlet.setCurrentMainScreen( DHISMIDlet.DATASET_MAINSCREEN );
                }
                else if ( dataSets.size() <= 0 && progs.size() > 0 )
                {
                    dhisMIDlet.setCurrentMainScreen( DHISMIDlet.ACTIVITY_MAINSCREEN );
                }

                if ( dhisMIDlet.isWaiting() )
                {
                    dhisMIDlet.setWaiting( false );
                    dhisMIDlet.saveInitSetting();
                    dhisMIDlet.showMainScreen();
                }

                progs = null;
                dataSets = null;
                mobileWrapper = null;
                dhisMIDlet.setDownloading( false );
                System.gc();
            }
            catch ( Exception e )
            {
                dhisMIDlet.switchDisplayable( Alerts.getErrorAlert( Text.ERROR(), "Error" ), dhisMIDlet.getLoginForm() );
                e.printStackTrace();
            }

        }

        else if ( task == ConnectionManager.UPDATE_ACTIVITYPLAN )
        {
            updateActivityPlan( orgUnit.getUpdateActivityPlanUrl() );
        }
        else if ( task == ConnectionManager.UPDATE_DATASET )
        {
            try
            {
                updateDataSet();

                if ( !(dataSetList.getAddedDataSets() == null && dataSetList.getDeletedDataSets() == null && dataSetList
                    .getModifiedDataSets() == null) )
                {
                    dhisMIDlet.saveUpdatedDataSet( dataSetList );
                    dhisMIDlet.loadDataSets( dhisMIDlet.getDataSetDisplayList() );
                    RefreshDataSetListConfirmListener listener = new RefreshDataSetListConfirmListener();
                    dhisMIDlet.getDisplay().setCurrent(
                        Alerts.getConfirmAlert( Text.UPDATE_DATASET(),
                            Text.UPDATE_FORMS() + updateDataSetResult.toString(), listener, dhisMIDlet, dhisMIDlet
                                .getDisplay().getCurrent(), dhisMIDlet.getDataSetDisplayList() ) );
                }
            }
            catch ( Exception e )
            {
                dhisMIDlet.setDownloading( false );
                e.printStackTrace();
            }
        }
    }

    private void updateActivityPlan( String updateURL )
    {
        ModelList currentProgramList = getAllCurrentPrograms();

        HttpConnection hcon = null;
        DataOutputStream dos = null;
        DataInputStream dis = null;

        try
        {
            int redirectTimes = 0;
            boolean redirect;
            do
            {
                redirect = false;

                hcon = (HttpConnection) Connector.open( updateURL );
                configureConnection( hcon );

                hcon.setRequestMethod( HttpConnection.POST );

                byte[] request_body = SerializationUtil.serialize( currentProgramList );
                hcon.setRequestProperty( "Content-Length", "" + request_body.toString().length() );
                dos = hcon.openDataOutputStream();

                for ( int i = 0; i < request_body.length; i++ )
                {
                    dos.writeByte( request_body[i] );
                }
                dos.flush();

                int status = hcon.getResponseCode();

                switch ( status )
                {
                case HttpConnection.HTTP_OK:
                    dis = hcon.openDataInputStream();
                    dis = getDecompressedStream( dis );
                    mobileWrapper = new MobileWrapper();
                    mobileWrapper.deSerialize( dis );

                    dhisMIDlet.saveActivityPlan( mobileWrapper.getActivityPlan() );

                    for ( int i = 0; i < mobileWrapper.getPrograms().size(); i++ )
                    {
                        dhisMIDlet.saveProgram( (Program) mobileWrapper.getPrograms().elementAt( i ) );
                    }
                    
                    dhisMIDlet.setServerCurrentDate( mobileWrapper.getServerCurrentDate() );
                    
                    mobileWrapper = null;
                    dhisMIDlet.setDownloading( false );
                    // dhisMIDlet.switchDisplayable( Alerts.getSuccessAlert(
                    // Text.SUCCESS(), Text.UPDATE_COMPLETED() ),
                    // dhisMIDlet.getMainMenuList() );
                    if ( dhisMIDlet.getDisplay().getCurrent() == dhisMIDlet.getVillageForms() )
                    {
                        dhisMIDlet.displayVillageForms();
                    }
                    return;
                case HttpConnection.HTTP_NO_CONTENT:
                case HttpConnection.HTTP_TEMP_REDIRECT:
                case HttpConnection.HTTP_MOVED_TEMP:
                case HttpConnection.HTTP_MOVED_PERM:

                    url = hcon.getHeaderField( "location" );

                    if ( hcon != null )
                        hcon.close();
                    redirectTimes++;
                    redirect = true;
                    break;
                case HttpsConnection.HTTP_CONFLICT:
                    handleError( "Problem with sent data", hcon );
                case HttpConnection.HTTP_INTERNAL_ERROR:
                    handleError( "Server encountered unexpected problem", hcon );
                default:
                    hcon.close();
                    handleError( "Communication with server failed", hcon );
                }

            }
            while ( redirect == true && redirectTimes < 5 );

            if ( redirectTimes == 5 )
            {
                throw new IOException( "Too many redirects" );
            }
        }
        catch ( Exception e )
        {
            System.out.println( e.getMessage() );
            e.printStackTrace();
            dhisMIDlet.setDownloading( false );

            if ( dhisMIDlet.getDisplay().getCurrent() == dhisMIDlet.getVillageForms() )
            {
                dhisMIDlet.displayVillageForms();
            }
        }
        finally
        {
            try
            {
                if ( hcon != null )
                    hcon.close();
                if ( dos != null )
                    dos.close();
            }
            catch ( Exception ioe )
            {
                ioe.printStackTrace();
            }
        }

    }

    private ModelList getAllCurrentPrograms()
    {
        ModelList currentProgramList = new ModelList();
        Vector programList = new Vector();
        ModelRecordStore programRecordStore = new ModelRecordStore( ModelRecordStore.PROGRAM_DB );
        Vector byteVector = null;
        try
        {
            byteVector = programRecordStore.getAllRecordAsByte();
            for ( int i = 0; i < byteVector.size(); i++ )
            {
                byte[] bite = (byte[]) byteVector.elementAt( i );
                Program program = new Program();
                SerializationUtil.deSerialize( program, bite );
                Model model = new Model();
                model.setId( program.getId() );

                // Use model name to store version
                model.setName( String.valueOf( program.getVersion() ) );
                programList.addElement( model );
            }
            currentProgramList.setModels( programList );
        }
        catch ( RecordStoreException e )
        {
            System.out.println( e.getMessage() );
            e.printStackTrace();
        }
        catch ( IOException e )
        {
            System.out.println( e.getMessage() );
            e.printStackTrace();
        }
        return currentProgramList;

    }

    private void uploadActivityReport()
    {
        Alert alert = null;
        List activityPlanList = dhisMIDlet.getActivityPlanList();

        try
        {
            upload( orgUnit.getUploadActivityReportUrl(), activityValue );
            activityPlanList.delete( activityPlanList.getSelectedIndex() );
            dhisMIDlet.updateActivityStatus();
            alert = Alerts.getSuccessAlert( Text.SUCCESS(), "Activity upploaded sucessfully" );
        }
        catch ( SecurityException e )
        {
            alert = Alerts.getErrorAlert( Text.UPLOAD_FAIL(),
                "Was not allowed to send activity report: " + e.getMessage() );
        }
        catch ( IOException e )
        {
            alert = Alerts.getErrorAlert( Text.UPLOAD_FAIL(), e.getMessage() );
            e.printStackTrace();
        }

        dhisMIDlet.refreshActivityPlan();
        dhisMIDlet.switchDisplayable( alert, activityPlanList );
    }

    private void uploadDataSetReport()
    {
        Alert alert = null;
        try
        {
            upload( orgUnit.getUploadFacilityReportUrl(), dataSetValue );
            dataSetValue.setCompleted( true );
            dhisMIDlet.saveDataSetValue( dataSetValue );
            alert = Alerts.getSuccessAlert( Text.SUCCESS(), Text.REPORT_SENT() );
        }
        catch ( IOException e )
        {
            String message = e.getMessage();

            if ( DHISMIDlet.DEBUG )
            {
                System.out.println( message );
                e.printStackTrace();
            }

            if ( message.equalsIgnoreCase( "DATASET_LOCKED" ) )
            {
                alert = Alerts.getErrorAlert( Text.UPLOAD_FAIL(), Text.DATASET_LOCKED() );
            }
            else
            {
                alert = Alerts.getErrorAlert( Text.UPLOAD_FAIL(), message );
            }
        }

        dhisMIDlet.switchDisplayable( alert, dhisMIDlet.getDataSetDisplayList() );

    }

    private void downloadAll()
        throws IOException
    {
        String url = orgUnit.getDownloadAllUrl();
        HttpConnection hcon = null;
        DataInputStream dis = null;
        try
        {
            int redirectTimes = 0;
            boolean redirect;
            do
            {
                redirect = false;
                hcon = (HttpConnection) Connector.open( url );
                configureConnection( hcon );
                int status = hcon.getResponseCode();
                switch ( status )
                {
                case HttpConnection.HTTP_OK:
                    dis = new DataInputStream( hcon.openInputStream() );
                    dis = getDecompressedStream( dis );

                    if ( dis != null )
                    {
                        mobileWrapper = new MobileWrapper();
                        mobileWrapper.deSerialize( dis );
                    }

                    dhisMIDlet.setDownloading( false );
                    break;
                case HttpConnection.HTTP_TEMP_REDIRECT:
                case HttpConnection.HTTP_MOVED_TEMP:
                case HttpConnection.HTTP_MOVED_PERM:

                    url = hcon.getHeaderField( "location" );

                    if ( dis != null )
                        dis.close();
                    if ( hcon != null )
                        hcon.close();

                    hcon = null;
                    redirectTimes++;
                    redirect = true;
                    break;
                default:
                    hcon.close();
                    throw new IOException( "Response status not OK:" + status );
                }
            }
            while ( redirect == true && redirectTimes < 5 );

            if ( redirectTimes == 5 )
            {
                throw new IOException( "Too many redirects" );
            }

        }
        catch ( ConnectionNotFoundException e )
        {
            dhisMIDlet.switchDisplayable( Alerts.getErrorAlert( Text.ERROR(), Text.SERVER_NOT_FOUND() ),
                dhisMIDlet.getLoginForm() );
            e.printStackTrace();
            dhisMIDlet.setDownloading( true );
        }
        finally
        {
            try
            {
                if ( hcon != null )
                    hcon.close();
                if ( dis != null )
                    dis.close();

            }
            catch ( IOException e )
            {
            }
        }
    }

    private void updateDataSet()
        throws IOException
    {
        String url = orgUnit.getUpdateDataSetsUrl();
        HttpConnection hcon = null;
        DataOutputStream dos = null;
        DataInputStream dis = null;
        int redirectTimes = 0;
        boolean redirect;
        do
        {
            redirect = false;

            hcon = (HttpConnection) Connector.open( url );
            configureConnection( hcon );

            hcon.setRequestMethod( HttpConnection.POST );

            byte[] request_body = SerializationUtil.serialize( dataSetList );
            hcon.setRequestProperty( "Content-Length", "" + request_body.toString().length() );
            dos = hcon.openDataOutputStream();

            for ( int i = 0; i < request_body.length; i++ )
            {
                dos.writeByte( request_body[i] );
            }
            dos.flush();

            int uploadStatus = hcon.getResponseCode();
            switch ( uploadStatus )
            {
            case HttpConnection.HTTP_NO_CONTENT:
            case HttpConnection.HTTP_TEMP_REDIRECT:
            case HttpConnection.HTTP_MOVED_TEMP:
            case HttpsConnection.HTTP_CONFLICT:
                handleError( "Problem with sent data", hcon );
            case HttpConnection.HTTP_INTERNAL_ERROR:
                handleError( "Server encountered unexpected problem", hcon );
            case HttpConnection.HTTP_MOVED_PERM:

                url = hcon.getHeaderField( "location" );

                if ( hcon != null )
                    hcon.close();

                redirectTimes++;
                redirect = true;
                break;
            case HttpConnection.HTTP_OK:
                dis = new DataInputStream( hcon.openInputStream() );
                dis = getDecompressedStream( dis );
                dataSetList = new DataSetList();

                dataSetList.deSerialize( dis );

                updateDataSetResult = new StringBuffer();
                if ( dataSetList.getAddedDataSets() != null )
                {
                    for ( int i = 0; i < dataSetList.getAddedDataSets().size(); i++ )
                    {
                        updateDataSetResult.append( "Added Form(s):\n" );
                        updateDataSetResult.append( ((DataSet) dataSetList.getAddedDataSets().elementAt( i )).getName()
                            + "\n" );
                    }
                }
                if ( dataSetList.getDeletedDataSets() != null )
                {
                    for ( int i = 0; i < dataSetList.getDeletedDataSets().size(); i++ )
                    {
                        updateDataSetResult.append( "Removed Form(s):\n" );
                        updateDataSetResult.append( ((DataSet) dataSetList.getDeletedDataSets().elementAt( i ))
                            .getName() + "\n" );
                    }
                }
                if ( dataSetList.getModifiedDataSets() != null )
                {
                    for ( int i = 0; i < dataSetList.getModifiedDataSets().size(); i++ )
                    {
                        updateDataSetResult.append( "Modified Form(s):\n" );
                        updateDataSetResult.append( ((DataSet) dataSetList.getModifiedDataSets().elementAt( i ))
                            .getName() + "\n" );
                    }
                }
                if ( updateDataSetResult.length() == 0 )
                {
                    updateDataSetResult = null;
                }

                if ( dhisMIDlet.getCurrentMainScreen() == DHISMIDlet.GENERAL_MAINSCREEN )
                {
                    this.updateActivityPlan( orgUnit.getUpdateActivityPlanUrl() );
                }

                break;
            default:
                handleError( "Connection with server fail", hcon );
                hcon.close();
            }
        }
        while ( redirect == true && redirectTimes < 5 );
        if ( redirectTimes == 5 )
        {
            throw new IOException( "Too many redirects" );
        }
        if ( hcon != null )
            hcon.close();
        if ( dos != null )
            dos.close();
    }

    private void authenticate()
        throws Exception
    {
        OrgUnit orgUnit = new OrgUnit();
        download( orgUnit );

    }

    private void download( DataStreamSerializable entity )
        throws IOException
    {
        HttpConnection hcon = null;

        try
        {
            int redirectTimes = 0;
            boolean redirect;
            do
            {
                redirect = false;
                hcon = (HttpConnection) Connector.open( url );
                configureConnection( hcon );

                int status = hcon.getResponseCode();
                switch ( status )
                {
                case HttpConnection.HTTP_OK:
                    handleDownloadOk( entity, hcon );
                    break;
                case HttpConnection.HTTP_SEE_OTHER:
                case HttpConnection.HTTP_TEMP_REDIRECT:
                case HttpConnection.HTTP_MOVED_TEMP:
                case HttpConnection.HTTP_MOVED_PERM:
                    url = hcon.getHeaderField( "location" );
                    if ( hcon != null )
                        hcon.close();
                    hcon = null;
                    redirectTimes++;
                    redirect = true;
                    break;
                case HttpConnection.HTTP_INTERNAL_ERROR:
                    hcon.close();
                    throw new IOException( "Server Error" );
                case HttpConnection.HTTP_UNAUTHORIZED:
                    hcon.close();
                    dhisMIDlet.getUserName().setString( "" );
                    dhisMIDlet.getPassword().setString( "" );
                    throw new IOException( "Invalid User Name or Password" );
                case HttpConnection.HTTP_NOT_FOUND:
                    hcon.close();
                    throw new IOException( "Connection Not Found" );
                default:
                    hcon.close();
                    throw new IOException( "Response status not OK:" + status );
                }

            }
            while ( redirect == true && redirectTimes < 5 );

            if ( redirectTimes == 5 )
            {
                throw new IOException( "Too much redirects" );
            }
        }
        finally
        {
            try
            {
                if ( hcon != null )
                    hcon.close();
            }
            catch ( IOException e )
            {
            }
        }

    }

    private void handleDownloadOk( DataStreamSerializable entity, HttpConnection hcon )
        throws IOException
    {
        DataInputStream dis = null;
        try
        {
            dis = new DataInputStream( hcon.openInputStream() );

            dis = getDecompressedStream( dis );

            int size;

            try
            {
                size = dis.readInt();
            }
            catch ( IOException e )
            {
                throw new IOException( "Problem reading response, wrong server address?" );
            }

            if ( size == 1 )
            {
                // Only one OruUnit, let's set OrgUnit for ConnectionManager
                entity.deSerialize( dis );
                ConnectionManager.orgUnit = (OrgUnit) entity;
            }
            else if ( size == 0 )
            {
                throw new IOException( "Not one org unit, but " + size );
            }
            else
            {
                // If there are more than one OrgUnit, navigate user to OrgUnit
                // Selection list
                for ( int i = 0; i < size; i++ )
                {
                    OrgUnit orgUnit = new OrgUnit();
                    orgUnit.deSerialize( dis );
                    dhisMIDlet.getOrgUnits().addElement( orgUnit );
                }
                dhisMIDlet.selectOrgUnit();
            }

        }
        finally
        {
            try
            {
                if ( dis != null )
                    dis.close();
            }
            catch ( IOException ioe )
            {
            }

        }
    }

    private void upload( String url, DataStreamSerializable entity )
        throws IOException
    {
        HttpConnection hcon = null;
        DataOutputStream dos = null;

        try
        {
            int redirectTimes = 0;
            boolean redirect;
            do
            {
                redirect = false;

                hcon = (HttpConnection) Connector.open( url );
                configureConnection( hcon );

                hcon.setRequestMethod( HttpConnection.POST );

                byte[] request_body = SerializationUtil.serialize( entity );
                hcon.setRequestProperty( "Content-Length", "" + request_body.toString().length() );
                dos = hcon.openDataOutputStream();

                for ( int i = 0; i < request_body.length; i++ )
                {
                    dos.writeByte( request_body[i] );
                }
                dos.flush();

                int status = hcon.getResponseCode();

                switch ( status )
                {
                case HttpConnection.HTTP_OK:
                case HttpConnection.HTTP_NO_CONTENT:
                    // Went fine!
                    return;
                case HttpConnection.HTTP_TEMP_REDIRECT:
                case HttpConnection.HTTP_MOVED_TEMP:
                case HttpConnection.HTTP_MOVED_PERM:

                    url = hcon.getHeaderField( "location" );

                    if ( hcon != null )
                        hcon.close();

                    redirectTimes++;
                    redirect = true;
                    break;
                case HttpsConnection.HTTP_CONFLICT:
                    handleError( "Problem with sent data", hcon );
                case HttpConnection.HTTP_INTERNAL_ERROR:
                    handleError( "Server encountered unexpected problem", hcon );
                default:
                    hcon.close();
                    handleError( "Communication with server failed", hcon );
                }

            }
            while ( redirect == true && redirectTimes < 5 );

            if ( redirectTimes == 5 )
            {
                throw new IOException( "Too many redirects" );
            }
        }
        finally
        {
            try
            {
                if ( hcon != null )
                    hcon.close();
                if ( dos != null )
                    dos.close();
            }
            catch ( IOException ioe )
            {
                ioe.printStackTrace();
            }
        }
    }

    private void handleError( String message, HttpConnection hcon )
        throws IOException
    {
        StringBuffer responseMessage = new StringBuffer();

        DataInputStream dis = null;
        try
        {
            dis = new DataInputStream( hcon.openInputStream() );
            int ch;
            while ( (ch = dis.read()) != -1 )
            {
                responseMessage.append( (char) ch );
            }
        }
        catch ( IOException e )
        {
        }
        finally
        {
            try
            {
                if ( dis != null )
                    dis.close();
            }
            catch ( IOException ioe )
            {
            }
        }

        message += ", " + hcon.getResponseCode();
        if ( responseMessage.length() > 0 )
        {
            throw new IOException( responseMessage.toString() );

        }
        else
        {
            throw new IOException( message );
        }

    }

    private void configureConnection( HttpConnection conn )
        throws IOException
    {

        conn.setRequestProperty( "User-Agent", ua );
        conn.setRequestProperty( "Accept-Language", locale );
        conn.setRequestProperty( "Content-Type", MEDIATYPE_MOBILE_SERIALIZED );
        conn.setRequestProperty( "Accept", MEDIATYPE_MOBILE_SERIALIZED );

        // set HTTP basic authentication
        if ( userName != null && password != null )
        {
            byte[] auth = (userName + ":" + password).getBytes();
            conn.setRequestProperty( "Authorization", "Basic " + Base64.encode( auth, 0, auth.length ) );
        }
    }

    private DataInputStream getDecompressedStream( DataInputStream dis )
    {
        return new DataInputStream( new ZInputStream( dis ) );
    }

    public static OrgUnit getOrgUnit()
    {
        return orgUnit;
    }

    public static void setOrgUnit( OrgUnit orgUnit )
    {
        ConnectionManager.orgUnit = orgUnit;
    }
}