package org.hisp.dhis.mobile.db;

/*
 * Copyright (c) 2004-2010, University of Oslo All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution. * Neither the name of the HISP project nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Date;

import javax.microedition.rms.RecordFilter;

import org.hisp.dhis.mobile.model.Activity;
import org.hisp.dhis.mobile.model.Period;

public class ActivityRecordFilter
    implements RecordFilter
{
    public static final int PROGRAM_STAGE_INSTANCE_ID = 0;

    public static final int STATUS_COMPLETE = 1;

    public static final int STATUS_INCOMPLETE = 2;

    public static final int STATUS_EXPIRED = 3;

    private int progStageInstId;

    private int filter;

    private Date currentDate;

    public ActivityRecordFilter( int filter )
    {
        if ( filter < 0 || filter > 2 )
            throw new IllegalArgumentException( "Unknown filter" );
        this.filter = filter;
    }

    public int getProgStageInstId()
    {
        return progStageInstId;
    }

    public void setProgStageInstId( int progStageInstId )
    {
        this.progStageInstId = progStageInstId;
    }

    public Date getCurrentDate()
    {
        return currentDate;
    }

    public void setCurrentDate( Date currentDate )
    {
        this.currentDate = currentDate;
    }

    public boolean matches( byte[] candidate )
    {
        if ( this.filter == STATUS_COMPLETE )
        {
            ByteArrayInputStream bis = new ByteArrayInputStream( candidate );
            DataInputStream dis = new DataInputStream( bis );

            try
            {
                dis.readInt();
                dis.readInt();
                dis.readInt();
                return dis.readBoolean();
            }
            catch ( Exception e )
            {
                System.out.println( "Activity Filter get exception" );
                e.printStackTrace();
                return false;
            }
            finally
            {
                try
                {
                    bis.close();
                    dis.close();
                }
                catch ( IOException e )
                {
                    e.printStackTrace();
                }

            }
        }
        else if ( this.filter == STATUS_INCOMPLETE )
        {
            ByteArrayInputStream bis = new ByteArrayInputStream( candidate );
            DataInputStream dis = new DataInputStream( bis );
            try
            {
                dis.readInt();
                dis.readInt();
                dis.readInt();
                return !dis.readBoolean();
            }
            catch ( Exception e )
            {
                System.out.println( "Activity Filter get exception" );
                e.printStackTrace();
                return false;
            }
            finally
            {
                try
                {
                    bis.close();
                    dis.close();
                }
                catch ( IOException e )
                {
                    e.printStackTrace();
                }

            }
        }
        else if ( this.filter == PROGRAM_STAGE_INSTANCE_ID )
        {
            ByteArrayInputStream bis = new ByteArrayInputStream( candidate );
            DataInputStream dis = new DataInputStream( bis );
            try
            {
                return (dis.readInt() == this.progStageInstId);
            }
            catch ( Exception e )
            {
                System.out.println( "Activity Filter get exception" );
                e.printStackTrace();
                return false;
            }
            finally
            {
                try
                {
                    bis.close();
                    dis.close();
                }
                catch ( IOException e )
                {
                    e.printStackTrace();
                }

            }
        }
        return false;
    }
}
