package org.hisp.dhis.mobile.db;

/*
 * Copyright (c) 2004-2010, University of Oslo All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution. * Neither the name of the HISP project nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.io.IOException;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreNotOpenException;

import org.hisp.dhis.mobile.model.Activity;
import org.hisp.dhis.mobile.model.Period;
import org.hisp.dhis.mobile.ui.DHISMIDlet;
import org.hisp.dhis.mobile.util.SerializationUtil;

public class ActivityRecordStore
    implements Runnable
{
    public static final String LOAD_ALL_ACTIVITYPLAN = "loadall";

    public static final String LOAD_CURRENT_ACTIVITYPLAN = "loadcurrent";

    public static final String LOAD_COMPLETED_ACTIVITYPLAN = "loadcompleted";

    public static final String ACTIVITY_DB = "ACTIVITY";

    private String task;

    private Vector activityVector;

    private DHISMIDlet dhisMIDlet;

    // Constructor
    public ActivityRecordStore( DHISMIDlet dhisMIDlet, String task )
    {
        this.dhisMIDlet = dhisMIDlet;
        this.task = task;
    }

    public ActivityRecordStore( DHISMIDlet dhisMIDlet )
    {
        this.dhisMIDlet = dhisMIDlet;
    }

    public ActivityRecordStore()
    {
    }

    // Getter & Setter

    public Vector getActivityVector()
    {
        return activityVector;
    }

    public void setActivityVector( Vector activityVector )
    {
        this.activityVector = activityVector;
    }

    // Supportive methods

    public void save()
        throws IOException
    {
        // clear();
        RecordStore rs = null;
        RecordEnumeration re = null;
        ActivityRecordFilter activityRecordFilter = null;
        // clear();

        try
        {
            rs = RecordStore.openRecordStore( ACTIVITY_DB, true );
            activityRecordFilter = new ActivityRecordFilter( ActivityRecordFilter.PROGRAM_STAGE_INSTANCE_ID );
            if ( activityVector != null && activityVector.size() > 0 )
            {
                Enumeration activities = activityVector.elements();

                Activity activity;
                while ( activities.hasMoreElements() )
                {
                    activity = (Activity) activities.nextElement();
                    activityRecordFilter.setProgStageInstId( activity.getTask().getProgStageInstId() );
                    re = rs.enumerateRecords( activityRecordFilter, null, false );
                    if ( re.numRecords() <= 0 )
                    {
                        if ( DHISMIDlet.DEBUG )
                            System.out.println( "Saving " + activity.getBeneficiary().getFullName() + " complete: "
                                + activity.getTask().isComplete() );
                        byte[] activityByte = SerializationUtil.serialize( activity );
                        rs.addRecord( activityByte, 0, activityByte.length );
                    }
                }
            }
        }
        catch ( RecordStoreException e )
        {
            e.printStackTrace();
        }
        finally
        {
            if ( rs != null )
                try
                {
                    rs.closeRecordStore();
                }
                catch ( RecordStoreNotOpenException e )
                {
                    e.printStackTrace();
                }
                catch ( RecordStoreException e )
                {
                    e.printStackTrace();
                }
            System.gc();
        }
    }

    public void clear()
    {
        RecordStore rs = null;
        RecordEnumeration re = null;
        try
        {
            rs = RecordStore.openRecordStore( ACTIVITY_DB, true );
            re = rs.enumerateRecords( null, null, false );
            int id;
            while ( re.hasNextElement() )
            {
                id = re.nextRecordId();
                rs.deleteRecord( id );
            }
        }
        catch ( Exception e )
        {

        }
        finally
        {
            if ( re != null )
                re.destroy();
            if ( rs != null )
                try
                {
                    rs.closeRecordStore();
                }
                catch ( RecordStoreNotOpenException e )
                {
                    e.printStackTrace();
                }
                catch ( RecordStoreException e )
                {
                    e.printStackTrace();
                }
        }
    }

    public void loadAll()
        throws IOException, RecordStoreException
    {
        load( new ActivityRecordFilter( ActivityRecordFilter.STATUS_INCOMPLETE ) );
    }

    public void loadCurrentActivityPlan()
        throws RecordStoreException, IOException
    {
        if ( DHISMIDlet.DEBUG )
            System.out.println( "load current" );
        load( new ActivityRecordFilter( ActivityRecordFilter.STATUS_INCOMPLETE ) );
    }

    public void loadCompletedActivityPlan()
        throws RecordStoreException, IOException
    {
        if ( DHISMIDlet.DEBUG )
            System.out.println( "load completed" );
        load( new ActivityRecordFilter( ActivityRecordFilter.STATUS_COMPLETE ) );
    }

    public void deleteExpiredActivities( Date currentDate )
    {
        ActivityValueRecordStore valueRecordStore = new ActivityValueRecordStore();
        ActivityRecordFilter filter = new ActivityRecordFilter( ActivityRecordFilter.STATUS_COMPLETE );
        RecordStore rs = null;
        RecordEnumeration re = null;
        activityVector = new Vector();
        try
        {
            rs = RecordStore.openRecordStore( ACTIVITY_DB, true );
            re = rs.enumerateRecords( filter, null, false );
            while ( re.hasNextElement() )
            {
                int id = re.nextRecordId();
                Activity activity = new Activity();
                SerializationUtil.deSerialize( activity,rs.getRecord( id ) );
                if ( Period.isExpired( currentDate, activity.getExpireDate() ) )
                {
                    valueRecordStore.delete( activity.getTask().getProgStageInstId() );
                    rs.deleteRecord( id );
                }
            }
            valueRecordStore = null;
            re = null;
            rs = null;
        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }
        finally
        {
            if ( re != null )
                re.destroy();
            if ( rs != null )
                try
                {
                    rs.closeRecordStore();
                }
                catch ( RecordStoreNotOpenException e )
                {
                    e.printStackTrace();
                }
                catch ( RecordStoreException e )
                {
                    e.printStackTrace();
                }
        }
    }

    private void load( ActivityRecordFilter filter )
        throws RecordStoreException, IOException
    {
        RecordStore rs = null;
        RecordEnumeration re = null;
        activityVector = new Vector();
        try
        {
            rs = RecordStore.openRecordStore( ACTIVITY_DB, true );
            re = rs.enumerateRecords( filter, null, false );
            while ( re.hasNextElement() )
            {
                Activity activity = new Activity();
                SerializationUtil.deSerialize( activity, re.nextRecord() );
                if ( DHISMIDlet.DEBUG )
                    System.out.println( activity.getBeneficiary().getFullName() + ", complete: "
                        + activity.getTask().isComplete() );
                activityVector.addElement( activity );
            }
        }
        finally
        {
            if ( re != null )
                re.destroy();
            if ( rs != null )
                try
                {
                    rs.closeRecordStore();
                }
                catch ( RecordStoreNotOpenException e )
                {
                    e.printStackTrace();
                }
                catch ( RecordStoreException e )
                {
                    e.printStackTrace();
                }
        }
    }

    public void updateActivityStatus( Activity activity )
        throws IOException
    {
        RecordStore rs = null;
        RecordEnumeration re = null;
        ActivityRecordFilter rf = new ActivityRecordFilter( ActivityRecordFilter.PROGRAM_STAGE_INSTANCE_ID );
        rf.setProgStageInstId( activity.getTask().getProgStageInstId() );
        try
        {
            rs = RecordStore.openRecordStore( ACTIVITY_DB, true );
            if ( activity != null )
            {
                re = rs.enumerateRecords( rf, null, true );
                if ( re.numRecords() == 1 )
                {
                    int id = re.nextRecordId();
                    byte[] activityBytes = SerializationUtil.serialize( activity );
                    rs.setRecord( id, activityBytes, 0, activityBytes.length );
                    if ( DHISMIDlet.DEBUG )
                        System.out.println( "Stored " + activity.getBeneficiary().getFullName() + " complete "
                            + activity.getTask().isComplete() );
                }
                else
                {
                    System.out
                        .println( re.numRecords() + " records with id " + activity.getTask().getProgStageInstId() );
                }
            }

        }
        catch ( RecordStoreException e )
        {
            e.printStackTrace();
        }
        finally
        {
            if ( rs != null )
                try
                {
                    rs.closeRecordStore();
                }
                catch ( RecordStoreNotOpenException e )
                {
                    e.printStackTrace();
                }
                catch ( RecordStoreException e )
                {
                    e.printStackTrace();
                }
            System.gc();
        }

    }

    public void run()
    {
        try
        {
            if ( task.equalsIgnoreCase( LOAD_COMPLETED_ACTIVITYPLAN ) )
            {
                loadCompletedActivityPlan();
                dhisMIDlet.loadActivityPlan( getActivityVector(), false );
                dhisMIDlet.fillActivityPlanList( null, false );
                dhisMIDlet.switchDisplayable( null, dhisMIDlet.getActivityPlanList() );
            }
            else if ( task.equalsIgnoreCase( LOAD_CURRENT_ACTIVITYPLAN ) )
            {
                loadCurrentActivityPlan();
                dhisMIDlet.loadActivityPlan( getActivityVector(), true );
                if ( dhisMIDlet.fillVillageForm( dhisMIDlet.getVillageForms() ) )
                {
                    dhisMIDlet.switchDisplayable( null, dhisMIDlet.getVillageForms() );
                    dhisMIDlet.setCurrentActivityFlag( true );
                }
                else
                {
                    dhisMIDlet.fillActivityPlanList( null, false );
                    dhisMIDlet.switchDisplayable( null, dhisMIDlet.getActivityPlanList() );
                    dhisMIDlet.setCurrentActivityFlag( false );
                }
                // } else if (task.equalsIgnoreCase(UPDATE_ACTIVITY)) {
                // updateActivity();
                //
                // }
                // loadAll();
                // dhisMIDlet.loadActivityPlan(getActivityVector());
            }
        }
        catch ( RecordStoreException e )
        {
            // FIXME
            e.printStackTrace();
        }
        catch ( IOException e )
        {
            // FIXME
            e.printStackTrace();
        }
        System.gc();
    }
}
