package org.hisp.dhis.mobile.db;

/*
 * Copyright (c) 2004-2010, University of Oslo All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution. * Neither the name of the HISP project nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.util.Vector;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import org.hisp.dhis.mobile.model.DataSet;
import org.hisp.dhis.mobile.model.DataSetValue;
import org.hisp.dhis.mobile.model.Period;
import org.hisp.dhis.mobile.ui.DHISMIDlet;
import org.hisp.dhis.mobile.ui.Text;
import org.hisp.dhis.mobile.util.SerializationUtil;

public class DataSetValueRecordStore
{
    public static final short STATE_NEW = 0;

    public static final short STATE_SAVED = 1;

    public static final short STATE_COMPLETED = 2;
    
    public static final short STATE_ERROR = 3;

    public static final String DATASETVALUE_DB = "DATASETVALUE";

    public static DataSetValueFilter dsValueFilter;

    public static String saveDataSetValue( DataSetValue dsValue )
    {

        RecordStore rs = null;
        RecordEnumeration re = null;
        getDSvalueFilter().clearFields();
        getDSvalueFilter().setDataSetID( dsValue.getId() );
        getDSvalueFilter().setPeriodName( dsValue.getpName() );
        try
        {
            rs = RecordStore.openRecordStore( DATASETVALUE_DB, true );
            re = rs.enumerateRecords( dsValueFilter, null, true );
            byte[] bytes = null;
            if ( re.numRecords() > 0 )
            {
                int id = re.nextRecordId();
                DataSetValue currentDataSet = new DataSetValue();
                SerializationUtil.deSerialize( currentDataSet, rs.getRecord( id ) );
                if ( currentDataSet.isCompleted() )
                {
                    dsValue.setCompleted( currentDataSet.isCompleted() );
                }
                bytes = SerializationUtil.serialize( dsValue );
                rs.setRecord( id, bytes, 0, bytes.length );
            }
            else
            {
                bytes = SerializationUtil.serialize( dsValue );
                rs.addRecord( bytes, 0, bytes.length );
            }
            rs.closeRecordStore();
            re = null;
            rs = null;
            dsValueFilter = null;
        }
        catch ( Exception e )
        {
            e.printStackTrace();
            return Text.SAVE_DATAVALUE_FAIL();
        }
        return Text.DATAVALUE_SAVED();
    }

    public static short getDataSetState( DataSet selectedDataSet, String periodName )
    {
        RecordStore rs = null;
        RecordEnumeration re = null;

        if ( selectedDataSet.getPeriodType().equals( "Monthly" ) )
        {
            periodName = Period.formatMonthlyPeriod( periodName );
        }
        else if ( selectedDataSet.getPeriodType().equals( "Weekly" ) )
        {
            periodName = Period.formatWeeklyPeriod( periodName );
        }

        getDSvalueFilter().clearFields();
        getDSvalueFilter().setDataSetID( selectedDataSet.getId() );
        getDSvalueFilter().setPeriodName( periodName );
        try
        {
            rs = RecordStore.openRecordStore( DATASETVALUE_DB, true );
            re = rs.enumerateRecords( dsValueFilter, null, true );
            if ( re.numRecords() > 0 )
            {
                DataSetValue dsValue = new DataSetValue();
                SerializationUtil.deSerialize( dsValue, re.nextRecord() );
                if ( dsValue.isCompleted() )
                    return STATE_COMPLETED;
                else
                    return STATE_SAVED;
            }
        }
        catch ( Exception e )
        {
            e.printStackTrace();
            return STATE_ERROR;
        }
        finally
        {
            if ( rs != null )
            {
                try
                {
                    rs.closeRecordStore();
                }
                catch ( RecordStoreException e )
                {
                }
            }
        }

        return STATE_NEW;
    }

    public static DataSetValue load( int dsID, String periodName )
    {
        DataSetValue dsValue = new DataSetValue();
        getDSvalueFilter().clearFields();
        getDSvalueFilter().setDataSetID( dsID );
        getDSvalueFilter().setPeriodName( periodName );
        RecordStore rs = null;
        RecordEnumeration re = null;

        try
        {
            rs = RecordStore.openRecordStore( DATASETVALUE_DB, true );
            re = rs.enumerateRecords( dsValueFilter, null, true );
            if ( re.numRecords() > 0 )
            {
                SerializationUtil.deSerialize( dsValue, re.nextRecord() );
                return dsValue;
            }
            rs.closeRecordStore();
            dsValueFilter = null;
            re = null;
            rs = null;

        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }
        return null;
    }

    public static void clearOldDataSetValue( int dsID, int periodType, Vector currentPeriods )
    {
        getDSvalueFilter().clearFields();
        getDSvalueFilter().setDataSetID( dsID );
        getDSvalueFilter().setPeriodType( periodType );
        getDSvalueFilter().setCurrentPeriods( currentPeriods );
        RecordStore rs = null;
        RecordEnumeration re = null;
        try
        {
            rs = RecordStore.openRecordStore( DATASETVALUE_DB, true );
            re = rs.enumerateRecords( dsValueFilter, null, true );

            if ( DHISMIDlet.DEBUG )
                System.out.println( "Number of value will be deleted:" + re.numRecords() );

            while ( re.hasNextElement() )
            {
                rs.deleteRecord( re.nextRecordId() );
            }
            if ( currentPeriods != null )
            {
                currentPeriods.removeAllElements();
                currentPeriods = null;
            }
        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }
    }

    public static DataSetValueFilter getDSvalueFilter()
    {
        if ( dsValueFilter == null )
        {
            dsValueFilter = new DataSetValueFilter();
        }
        return dsValueFilter;
    }

}
