package org.hisp.dhis.mobile.db;

/*
 * Copyright (c) 2004-2010, University of Oslo All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution. * Neither the name of the HISP project nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import javax.microedition.rms.RecordComparator;
import javax.microedition.rms.RecordStoreException;

import org.hisp.dhis.mobile.model.Model;

public class ModelComparator
    implements RecordComparator
{

    public final static int SORT_BY_ID = 0;

    public final static int SORT_BY_NAME = 1;

    private int sortOrder = SORT_BY_ID;

    public ModelComparator( int sortOrder )
    {
        if ( sortOrder != SORT_BY_ID && sortOrder != SORT_BY_NAME )
        {
            throw new IllegalArgumentException("sortOrder " + sortOrder + "doesn't exist");
        }
        
        this.sortOrder = sortOrder;
    }

    public int compare( byte[] rec1, byte[] rec2 )
    {
        Model comp1 = null;
        Model comp2 = null;

        try
        {
            comp1 = Model.fromBytes( rec1 );
            comp2 = Model.fromBytes( rec2 );
        }
        catch ( RecordStoreException e )
        {
            System.out.println("Oops.. Didn't expect that");
            e.printStackTrace();
            // FIXME
            return 0;
        }

        if ( sortOrder == SORT_BY_NAME )
        {

            String name1 = comp1.getName().toLowerCase();
            String name2 = comp2.getName().toLowerCase();

            int result = name1.compareTo( name2 );
            if ( result == 0 )
                return RecordComparator.EQUIVALENT;
            else if ( result < 0 )
                return RecordComparator.PRECEDES;
            else
                return RecordComparator.FOLLOWS;
        }
        else
        {
            if ( comp1.getId() == comp2.getId() )
                return RecordComparator.EQUIVALENT;
            else if ( comp1.getId() < comp2.getId() )
                return RecordComparator.PRECEDES;
            else
                return RecordComparator.FOLLOWS;
        }
    }
}