package org.hisp.dhis.mobile.db;

/*
 * Copyright (c) 2004-2010, University of Oslo All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution. * Neither the name of the HISP project nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.io.IOException;
import java.util.Vector;

import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;

import org.hisp.dhis.mobile.model.Model;

public class ValueRecordStore {

	public static final String VALUE_DB = "DATA_VALUE";

	private String dbName;

	public ValueRecordStore(String dbName) {
		this.dbName = dbName;
	}

	public byte[] getRecord(int id) throws RecordStoreException, IOException {
		RecordStore rs = null;
		RecordEnumeration re = null;
		byte nextRec[] = null;
		try {
			rs = RecordStore.openRecordStore(dbName, true);
			ModelComparator comparator = new ModelComparator(
					ModelComparator.SORT_BY_ID);
			re = rs.enumerateRecords(null, comparator, false);
			while (re.hasNextElement()) {
				nextRec = re.nextRecord();
				if (Model.fromBytes(nextRec).getId() == id)
					return nextRec;
			}
		} finally {
			if (re != null)
				re.destroy();
			if (rs != null)
				rs.closeRecordStore();
		}
		return null;
	}

	public byte[] getRecord(Model model) throws RecordStoreException {
		RecordStore rs = null;
		RecordEnumeration re = null;
		try {
			rs = RecordStore.openRecordStore(dbName, true);
			ModelFilter filter = new ModelFilter(model);
			ModelComparator comparator = new ModelComparator(
					ModelComparator.SORT_BY_ID);
			re = rs.enumerateRecords(filter, comparator, false);
			while (re.hasNextElement()) {
				return re.nextRecord();
			}
		} finally {
			if (re != null)
				re.destroy();
			if (rs != null)
				rs.closeRecordStore();
		}
		return null;
	}

	public void AddRecord(byte[] rec) throws RecordStoreException {
		RecordStore rs = null;
		RecordEnumeration re = null;
		try {
			rs = RecordStore.openRecordStore(dbName, true);
			Model model = Model.fromBytes(rec);
			ModelFilter filter = new ModelFilter(model);
			ModelComparator comparator = new ModelComparator(
					ModelComparator.SORT_BY_ID);
			re = rs.enumerateRecords(filter, comparator, false);
			while (re.hasNextElement()) {
				int id = re.nextRecordId();
				rs.deleteRecord(id);
			}

			rs.addRecord(rec, 0, rec.length);

		} finally {
			if (rs != null)
				rs.closeRecordStore();
		}
	}

	// The get all record method
	public Vector getAllRecord() throws RecordStoreException {
		RecordStore rs = null;
		RecordEnumeration re = null;
		Vector vector = new Vector();
		byte nextRec[] = null;
		try {
			rs = RecordStore.openRecordStore(dbName, true);
			ModelComparator comparator = new ModelComparator(
					ModelComparator.SORT_BY_NAME);
			re = rs.enumerateRecords(null, comparator, false);
			while (re.hasNextElement()) {
				nextRec = re.nextRecord();
				vector.addElement((Model.fromBytes( nextRec)));
			}
		} finally {
			if (re != null)
				re.destroy();
			if (rs != null)
				rs.closeRecordStore();
		}
		return vector;
	}

	public void deleteRecord(Model model) throws RecordStoreException {
		RecordStore rs = null;
		RecordEnumeration re = null;
		try {
			rs = RecordStore.openRecordStore(dbName, true);
			ModelFilter filter = new ModelFilter(model);
			ModelComparator comparator = new ModelComparator(
					ModelComparator.SORT_BY_ID);
			re = rs.enumerateRecords(filter, comparator, false);
			while (re.hasNextElement()) {
				int id = re.nextRecordId();
				rs.deleteRecord(id);
			}
		} finally {
			if (re != null)
				re.destroy();
			if (rs != null)
				rs.closeRecordStore();
		}

	}
}