package org.hisp.dhis.mobile.ui;

/*
 * Copyright (c) 2004-2010, University of Oslo All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution. * Neither the name of the HISP project nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.io.IOException;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.DateField;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.TextField;
import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.RecordStoreException;

import org.hisp.dhis.mobile.connection.ConnectionManager;
import org.hisp.dhis.mobile.db.ActivityRecordStore;
import org.hisp.dhis.mobile.db.ActivityValueRecordStore;
import org.hisp.dhis.mobile.db.DataSetRecordStore;
import org.hisp.dhis.mobile.db.DataSetValueRecordStore;
import org.hisp.dhis.mobile.db.ModelRecordStore;
import org.hisp.dhis.mobile.db.OrgUnitStore;
import org.hisp.dhis.mobile.db.SettingsRecordStore;
import org.hisp.dhis.mobile.model.Activity;
import org.hisp.dhis.mobile.model.ActivityPlan;
import org.hisp.dhis.mobile.model.ActivityValue;
import org.hisp.dhis.mobile.model.Beneficiary;
import org.hisp.dhis.mobile.model.DataElement;
import org.hisp.dhis.mobile.model.DataSet;
import org.hisp.dhis.mobile.model.DataSetList;
import org.hisp.dhis.mobile.model.DataSetValue;
import org.hisp.dhis.mobile.model.DataValue;
import org.hisp.dhis.mobile.model.Model;
import org.hisp.dhis.mobile.model.OrgUnit;
import org.hisp.dhis.mobile.model.PatientAttribute;
import org.hisp.dhis.mobile.model.PatientIdentifier;
import org.hisp.dhis.mobile.model.Period;
import org.hisp.dhis.mobile.model.Program;
import org.hisp.dhis.mobile.model.ProgramStage;
import org.hisp.dhis.mobile.model.Section;
import org.hisp.dhis.mobile.model.TextMapping;
import org.hisp.dhis.mobile.util.ReinitConfirmListener;
import org.hisp.dhis.mobile.util.SerializationUtil;

public class DHISMIDlet
    extends MIDlet
    implements CommandListener
{
    public static final boolean DEBUG = false;

    // Constants

    private static final String DEFAULT_DHIS_URL = "http://dhis.uio.no/test2";

    private static final String MOBILE_PATH = "api/mobile/";

    private static final String LOGO_IMAGE = "/dhis2_logo.png";

    public static final short GENERAL_MAINSCREEN = 0;

    public static final short ACTIVITY_MAINSCREEN = 1;

    public static final short DATASET_MAINSCREEN = 2;

    public static final String DEFAULT_LOCALE = "en-US";

    private int currentMainScreen = GENERAL_MAINSCREEN;

    private Image logo;

    Vector dataSetsVector = new Vector();

    Vector programsVector = new Vector();

    Vector activitiesVector = new Vector();

    Vector orgUnits = new Vector();

    // Supportive Variables

    private DataSet selectedDataSet;

    private Activity selectedActivity;

    private ProgramStage selectedProgramStage;

    private String selectedPeriod;

    public int currentSection;

    public Vector formVector;

    private Hashtable dataElements = new Hashtable();
    
    private Hashtable dateFieldValues = new Hashtable();

    private Date serverCurrentDate;

    // Flags

    private boolean downloading = true;

    private boolean waiting = false;

    private boolean midletPaused = false;

    private boolean currentActivity = true;

    private ChoiceGroup choiceGroup;

    // Login Screen

    private Form loginForm;

    private TextField url;

    private TextField userName;

    private TextField password;

    private Command lgnFrmExtCmd;

    private Command lgnFrmLgnCmd;

    private List orgUnitSelectList;

    // PIN Screen

    private Form pinForm;

    private TextField pinTextField;

    private Command exitCommand;

    private Command pinFormNextCmd;

    private Command pinFormReinitCmd;

    // Setting Screen

    private Form settingsForm;

    private TextField serverURL;

    private TextField dhisUserPass;

    private TextField dhisUserName;

    private TextField locale;

    private Command stngsOkCmd;

    private Command setngsBakCmd;

    private Command setngsSaveCmd;

    // Period Screen

    private Form periodForm;

    private DateField dailyPeriodDateField;

    private ChoiceGroup periodChoice;

    private Command selectDailyPeriodCmd;

    private Command periodBakCmd;

    private Command periodNxtCmd;

    // Wait Screens

    private Form waitForm;

    // Main Menu Screen

    private List mainMenuList;

    private Command mnuListDnldCmd;

    private Command mnuListExtCmd;

    private Command mnuListBackCmd;

    // Service Screen

    private List servicesList;

    private Command srvcsExitCmd;

    // DataSet Delete List

    private List deleteList;

    private Command deleteBakCmd;

    private Command dsDeleteCmd;

    // DataSet List Screen

    private List dataSetDisplayList;

    private Command dsLstBakCmd;

    private Command dsLstExitCmd;

    private Command dsDnldListBakCmd;

    // DataEntry (DataSet) Screen

    private Form dataEntryForm;

    private Command deFrmBakCmd;

    private Command deFrmSndCmd;

    private Command deFrmSavCmd;

    // Activity Detail Screen

    private Form activityDetailForm;

    private Command activityDetailCmd;

    private Command activityDetailOkCmd;

    // Activity Plan Screen

    private List activityPlanList;

    private Command actvPlnLstBakCmd;

    // Uneditable Screen

    private Form uneditableForm;

    private Command uneditableBckCmd;

    private Command dataSetUneditableBckCmd;

    // Villages List Screen

    private List villageForm;

    private Command villageBackCmd;

    private Vector attributes = new Vector();

    // Activity Entry Screen

    private Form activityEntryForm;

    private Command activityFrmBakCmd;

    private Command activityFrmSndCmd;

    private Command activityFrmSavCmd;

    // Delete Program Screen

    private List deleteProgList;

    private Command frmDnldCmd;

    // Dynamic DataSet Entry Screen

    private Command dsFormNextCmd;

    private Command dsFormBackCmd;

    private Command dsFormCompleteCmd;

    private Command dsFormExitCmd;

    private Command dsFormSaveCmd;

    private Command dsFormEditCmd;

    // Other Commands

    private Command screenCommand;

    private Command backCommand;

    private Command okCommand;

    private Command settingCommand;

    private OrgUnit orgUnit;

    private Command updateCommand;

    // For text value
    private static final int TEXT_NUMERIC = 6;

    private static final int TEXT_STRING = 32;

    public void startMIDlet()
    {
        orgUnit = OrgUnitStore.getOrgUnit();

        SettingsRecordStore settingStore = null;
        Form next = null;

        try
        {
            settingStore = new SettingsRecordStore( SettingsRecordStore.SETTINGS_DB );
            if ( settingStore.get( "pin" ).equals( "" ) )
            {
                next = getLoginForm();
            }
            else
            {
                this.getPinForm().append( Text.CHECK_PIN_SUGGESTION() );
                this.getPinForm().addCommand( this.getPinFormReinitCmd() );
                next = getPinForm();
            }
        }
        catch ( RecordStoreException e )
        {
            e.printStackTrace();
        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }

        new SplashScreen( getLogo(), getDisplay(), next );
    }

    /**
     * Performs an action assigned to the Mobile Device - MIDlet Resumed point.
     */
    public void resumeMIDlet()
    {
    }

    /**
     * Switches a current displayable in a display. The <code>display</code>
     * instance is taken from <code>getDisplay</code> method. This method is
     * used by all actions in the design for switching displayable.
     * 
     * @param alert the Alert which is temporarily set to the display; if
     *        <code>null</code>, then <code>nextDisplayable</code> is set
     *        immediately
     * @param nextDisplayable the Displayable to be set
     */
    public void switchDisplayable( Alert alert, Displayable nextDisplayable )
    {
        if ( alert == null )
        {
            getDisplay().setCurrent( nextDisplayable );
        }
        else
        {
            getDisplay().setCurrent( alert, nextDisplayable );
        }
    }

    /**
     * Called by a system to indicated that a command has been invoked on a
     * particular displayable.
     * 
     * @param command the Command that was invoked
     * @param displayable the Displayable where the command was invoked
     */
    public void commandAction( Command command, Displayable displayable )
    {
        if ( displayable == dataEntryForm )
        {
            if ( command == deFrmBakCmd )
            {
                switchDisplayable( null, getDataSetDisplayList() );
            }
            else if ( command == deFrmSndCmd )
            {
                sendDataSetValue();
            }
            else if ( command == deFrmSavCmd )
            {
                DataSetValue dsValue = generateDataSetValue();
                Display.getDisplay( this )
                    .setCurrent( Alerts.getInfoAlert( Text.STATUS(), saveDataSetValue( dsValue ) ) );
            }
        }
        if ( displayable == activityEntryForm )
        {
            if ( command == activityFrmBakCmd )
            {
                switchDisplayable( null, getActivityPlanList() );
            }
            else if ( command == activityFrmSndCmd )
            {
                sendActivityValue();
            }
            else if ( command == activityFrmSavCmd )
            {
                switchDisplayable( Alerts.getInfoAlert( Text.STATUS(), this.saveActivityValue() ), getActivityPlanList() );
//                Display.getDisplay( this ).setCurrent( Alerts.getInfoAlert( Text.STATUS(), this.saveActivityValue() ) );
            }
        }
        else if ( displayable == dataSetDisplayList )
        {
            if ( command == List.SELECT_COMMAND )
            {
                dataSetDisplayListAction();
            }
            else if ( command == dsLstBakCmd )
            {
                switchDisplayable( null, getServicesList() );
            }
            else if ( command == dsLstExitCmd )
            {
                exitMIDlet();
            }
            else if ( command == settingCommand )
            {
                switchDisplayable( null, getSettingsForm() );
            }
            else if ( command == updateCommand )
            {
                // Update DataSet
                this.getWaitForm().deleteAll();
                this.getWaitForm().setTitle( Text.CHECKING_FOR_UPDATE() );
                this.getWaitForm().append( Text.PLEASE_WAIT() );
                this.switchDisplayable( null, this.getWaitForm() );
                this.setWaiting( true );

                this.loadSettings();
                DataSetRecordStore dataSetRs = new DataSetRecordStore();
                try
                {
                    Vector currentDataSets = dataSetRs.getAllDataSet();

                    ConnectionManager.updateDataSets( currentDataSets );
                }
                catch ( RecordStoreException e )
                {
                    e.printStackTrace();
                }

            }
        }
        else if ( displayable == loginForm )
        {
            if ( command == lgnFrmExtCmd )
            {
                exitMIDlet();
            }
            else if ( command == lgnFrmLgnCmd )
            {
                if ( getUserName().getString().trim().length() == 0 || getPassword().getString().trim().length() == 0 )
                {
                    switchDisplayable( Alerts.getErrorAlert( Text.INCOMPLETE_FORM(), Text.UP_MISSING() ),
                        getLoginForm() );
                    return;
                }
                switchDisplayable( null, getWaitForm() );

                login();
            }
        }
        else if ( displayable == mainMenuList )
        {
            if ( command == List.SELECT_COMMAND )
            {
                mainMenuListAction();
            }
            else if ( command == mnuListExtCmd )
            {
                exitMIDlet();
            }
            else if ( command == mnuListBackCmd )
            {
                switchDisplayable( null, getServicesList() );
            }
            else if ( command == settingCommand )
            {
                this.loadSettings();
                this.switchDisplayable( null, getSettingsForm() );
            }
            else if ( command == updateCommand )
            {
                // currently, only update dataset is available
            }
        }
        else if ( displayable == servicesList )
        {
            if ( command == List.SELECT_COMMAND )
            {
                servicesListAction();
            }
            else if ( command == srvcsExitCmd )
            {
                exitMIDlet();
            }
            else if ( command == settingCommand )
            {
                switchDisplayable( null, getSettingsForm() );
            }
        }
        else if ( displayable == activityPlanList )
        {
            if ( command == List.SELECT_COMMAND )
            {
                activityListAction();
            }
            else if ( command == actvPlnLstBakCmd )
            {
                if ( this.currentActivity )
                {
                    switchDisplayable( null, getVillageForms() );
                }
                else
                {
                    switchDisplayable( null, getMainMenuList() );
                }
            }
        }
        else if ( displayable == activityDetailForm )
        {
            if ( command == activityDetailOkCmd )
            {
                if ( !selectedActivity.getTask().isComplete() )
                {
                    displayCurrentActivity( selectedProgramStage, getActivityEntryForm() );
                }
                else
                {
                    displayCompletedActivity( selectedProgramStage, getUneditableForm() );
                }

            }
        }
        else if ( displayable == periodForm )
        {
            if ( command == periodBakCmd )
            {
                switchDisplayable( null, getDataSetDisplayList() );
            }
            else if ( command == selectDailyPeriodCmd )
            {
                getDisplay().setCurrent( new CalendarCanvas( this ) );
            }
            else if ( command == periodNxtCmd )
            {
                switchDisplayable( null, getDataEntryForm() );
                String periodName = null;

                if ( !selectedDataSet.getPeriodType().equals( "Daily" ) )
                {
                    periodName = getPeriodChoice().getString( getPeriodChoice().getSelectedIndex() );
                    if ( periodName.indexOf( " * " ) != -1 )
                    {
                        periodName = periodName.substring( 0, periodName.indexOf( " * " ) );
                    }
                }

                if ( selectedDataSet.getPeriodType().equals( "Daily" ) )
                {
                    selectedPeriod = Period.formatDate( dailyPeriodDateField.getDate() );
                }
                else if ( selectedDataSet.getPeriodType().equals( "Weekly" ) )
                {
                    selectedPeriod = Period.formatWeeklyPeriod( periodName );
                }
                else if ( selectedDataSet.getPeriodType().equals( "Monthly" ) )
                {
                    selectedPeriod = Period.formatMonthlyPeriod( periodName );
                }
                else
                {
                    selectedPeriod = periodName;
                }

                final DataSetValue dsValue = DataSetValueRecordStore.load( selectedDataSet.getId(), selectedPeriod );

                currentSection = 0;
                getWaitForm().deleteAll();
                getWaitForm().setTitle( Text.LOADING() );
                getWaitForm().append( Text.PLEASE_WAIT() );
                switchDisplayable( null, getWaitForm() );

                if ( dsValue == null || dsValue.isCompleted() == false )
                {
                    new Thread( new Runnable()
                    {
                        public void run()
                        {
                            displayDataSetEntry( selectedDataSet, dsValue );
                        }
                    } ).start();

                }
                else if ( dsValue.isCompleted() )
                {
                    this.displayUneditableDataSetEntry( selectedDataSet, dsValue );
                }

            }
        }
        else if ( displayable == settingsForm )
        {
            if ( command == setngsBakCmd )
            {
                if ( currentMainScreen == GENERAL_MAINSCREEN )
                {
                    switchDisplayable( null, getServicesList() );
                }
                else if ( currentMainScreen == ACTIVITY_MAINSCREEN )
                {
                    switchDisplayable( null, getMainMenuList() );
                }
                else
                {
                    switchDisplayable( null, getDataSetDisplayList() );
                }
            }
            else if ( command == setngsSaveCmd )
            {
                Displayable nextScreen = null;
                if ( currentMainScreen == GENERAL_MAINSCREEN )
                {
                    nextScreen = getServicesList();
                }
                else if ( currentMainScreen == ACTIVITY_MAINSCREEN )
                {
                    nextScreen = getMainMenuList();
                }
                else
                {
                    nextScreen = getDataSetDisplayList();
                }
                saveSettings();
                switchDisplayable( Alerts.getInfoAlert( Text.STATUS(), Text.SETTING_SAVED_MESSAGE() ), nextScreen );
            }
        }
        else if ( displayable == villageForm )
        {
            if ( command == List.SELECT_COMMAND )
            {
                this.prepairToLoadActivityPlan();

                new Thread( new Runnable()
                {
                    public void run()
                    {
                        fillActivityPlanList(
                            (PatientAttribute) attributes.elementAt( villageForm.getSelectedIndex() ), true );

                        switchDisplayable( null, activityPlanList );

                    }
                } ).start();
            }
            else if ( command == villageBackCmd )
            {
                switchDisplayable( null, getMainMenuList() );
            }

        }
        else if ( displayable == pinForm )
        {
            if ( command == pinFormNextCmd )
            {
                checkPIN();
            }
            else if ( command == exitCommand )
            {
                exitMIDlet();
            }
            else if ( command == pinFormReinitCmd )
            {
                ReinitConfirmListener listener = new ReinitConfirmListener();
                listener.setCurrentScrren( getPinForm() );
                listener.setNextScreen( getLoginForm() );
                this.getDisplay().setCurrent(
                    Alerts.getConfirmAlert( Text.REINIT(), Text.REINIT_MESSAGE(), listener, this, getPinForm(),
                        getLoginForm() ) );
            }
        }
        else if ( displayable == uneditableForm )
        {
            if ( command == uneditableBckCmd )
            {
                this.switchDisplayable( null, getActivityPlanList() );
            }
            else if ( command == dataSetUneditableBckCmd )
            {
                this.loadDataSets( getDataSetDisplayList() );
                this.switchDisplayable( null, getDataSetDisplayList() );
            }
        }
        else if ( command == dsFormNextCmd )
        {
            currentSection += 1;
            switchDisplayable( null, (Form) formVector.elementAt( currentSection ) );
        }
        else if ( command == dsFormBackCmd )
        {
            currentSection -= 1;
            switchDisplayable( null, (Form) formVector.elementAt( currentSection ) );
        }
        else if ( command == dsFormExitCmd )
        {
            formVector = null;
            this.loadDataSets( this.getDataSetDisplayList() );
            this.switchDisplayable( null, this.getDataSetDisplayList() );
        }
        else if ( command == dsFormCompleteCmd )
        {
            this.sendDataSetValue();
        }
        else if ( command == dsFormSaveCmd )
        {
            DataSetValue dsValue = this.generateDataSetValue();
            dsValue.setCompleted( false );
            Display.getDisplay( this ).setCurrent( Alerts.getInfoAlert( Text.STATUS(), saveDataSetValue( dsValue ) ) );
        }
        else if ( command == dsFormEditCmd )
        {
            formVector = null;
            final DataSetValue dsValue = DataSetValueRecordStore.load( selectedDataSet.getId(), selectedPeriod );
            switchDisplayable( null, getWaitForm() );
            new Thread( new Runnable()
            {

                public void run()
                {
                    displayDataSetEntry( selectedDataSet, dsValue );
                }
            } ).start();

        }
        else if ( displayable == orgUnitSelectList )
        {
            if ( command == List.SELECT_COMMAND )
            {
                this.orgUnit = (OrgUnit) orgUnits.elementAt( this.getOrgUnitSelectList().getSelectedIndex() );
                OrgUnitStore.setOrgUnit( this.orgUnit );
                this.getPinForm().append( Text.CREATE_PIN_SUGGESTION() );
                this.switchDisplayable( null, this.getPinForm() );
                this.setDownloading( true );
                ConnectionManager.setOrgUnit( orgUnit );
                ConnectionManager.downloadAllResources();

            }
        }
    }

    // Display a completed Data Set (Uneditable Form)
    private void displayUneditableDataSetEntry( DataSet ds, DataSetValue dsValue )
    {
        selectedDataSet = ds;
        if ( ds == null )
        {
            loadDataSets( this.getDataSetDisplayList() );
            this.switchDisplayable( Alerts.getInfoAlert( Text.ERROR(), Text.FORM_UNAVAILABLE() ),
                getDataSetDisplayList() );
        }
        else
        {
            formVector = new Vector();
            // Loop through all Section of the DataSet
            int sectionsSize = ds.getSections().size();
            StringBuffer buffer = new StringBuffer();
            for ( int i = 0; i < sectionsSize; i++ )
            {
                Section section = (Section) ds.getSections().elementAt( i );
                Vector des = section.getDataElements();
                Form dsForm = new Form( section.getName() );
                // Loop through all DataElement of each Section
                int dataElementsSize = des.size();
                for ( int j = 0; j < dataElementsSize; j++ )
                {
                    DataElement de = (DataElement) des.elementAt( j );
                    Vector categoryOptCombos = de.getCategoryOptionCombos().getModels();
                    dsForm.append( de.getName() );
                    // Loop through all categoryOptCombo of each DataElement
                    int categoryOptCombosSize = categoryOptCombos.size();
                    for ( int k = 0; k < categoryOptCombosSize; k++ )
                    {
                        Model categoryOptCombo = (Model) categoryOptCombos.elementAt( k );
                        buffer = new StringBuffer();
                        buffer.append( de.getId() );
                        buffer.append( "-" );
                        buffer.append( categoryOptCombo.getId() );

                        DataValue dataValue = (DataValue) dsValue.getDataValues().get( buffer.toString().trim() );
                        String label = null;
                        if ( categoryOptCombosSize > 1 )
                        {
                            label = categoryOptCombo.getName();
                        }

                        TextField txtField = new TextField( label, "", 32, TextField.ANY | TextField.UNEDITABLE );
                        if ( de.getType().equals( "bool" ) )
                        {
                            if ( dataValue != null )
                                if ( dataValue.getVal().equalsIgnoreCase( "true" ) )
                                    txtField.setString( Text.YES() );
                                else if ( dataValue.getVal().equalsIgnoreCase( "false" ) )
                                    txtField.setString( Text.NO() );
                                else
                                    txtField.setString( Text.PLEASE_SELECT() );
                        }
                        else
                        {
                            if ( dataValue != null )
                                txtField.setString( dataValue.getVal() );

                        }
                        dsForm.append( txtField );
                        buffer = null;
                        System.gc();
                    }

                    // Add Command for Form
                    dsForm.setCommandListener( this );
                    dsForm.addCommand( getDsFormEditCmd() );
                    if ( i == 0 )
                    {
                        if ( ds.getSections().size() > 1 )
                        {
                            dsForm.addCommand( getDsFormNextCmd() );
                        }
                        dsForm.addCommand( getDsFormExitCmd() );
                    }
                    if ( i > 0 && i < ds.getSections().size() - 1 )
                    {
                        dsForm.addCommand( getDsFormNextCmd() );
                        dsForm.addCommand( getDsFormBackCmd() );
                    }
                    if ( i == ds.getSections().size() - 1 )
                    {
                        if ( ds.getSections().size() > 1 )
                        {
                            dsForm.addCommand( getDsFormBackCmd() );
                        }
                    }

                    de = null;
                }
                formVector.addElement( dsForm );
                dsForm = null;
                section = null;
                System.gc();
            }
        }
        switchDisplayable( null, (Form) formVector.elementAt( 0 ) );

    }

    private Command getDsFormEditCmd()
    {
        if ( dsFormEditCmd == null )
        {
            dsFormEditCmd = new Command( "Edit", Command.SCREEN, 1 );
        }
        return dsFormEditCmd;
    }

    private void checkPIN()
    {
        SettingsRecordStore settingRs = null;
        try
        {
            settingRs = new SettingsRecordStore( SettingsRecordStore.SETTINGS_DB );
            if ( settingRs.get( "pin" ).equals( "" ) )
            {
                this.saveInitSetting();
                if ( getPinTextField().getString().trim().length() == 4 )
                {

                    if ( this.isDownloading() )
                    {
                        // Wait until downloading process finished.
                        getWaitForm().deleteAll();
                        getWaitForm().append( Text.DOWNLOADING() );
                        switchDisplayable( null, getWaitForm() );
                        this.setWaiting( true );
                    }
                    else
                    {
                        // Display mainscreen
                        this.saveInitSetting();
                        this.showMainScreen();
                    }

                    // this.downloadAll();
                }
                else
                {
                    switchDisplayable( Alerts.getInfoAlert( Text.ERROR(), Text.EMPTY_PIN() ), getPinForm() );
                }
            }
            else
            {
                if ( settingRs.get( "pin" ).equals( getPinTextField().getString() ) )
                {
                    loadSettings();
                    initConnectionManager();
                    // Display current DataSet List
                    showMainScreen();
                    // Check for updates and redisplay the new DataSet List
                    if ( currentMainScreen == ACTIVITY_MAINSCREEN )
                    {
                        updateActivityPlan();
                    }
                    else
                    {
                        updateDataSets();
                    }

                }
                else
                {
                    switchDisplayable( Alerts.getInfoAlert( Text.ERROR(), Text.INVALID_PIN() ), getPinForm() );
                }
            }
            settingRs = null;
        }
        catch ( RecordStoreException e )
        {
            e.printStackTrace();
        }
    }

    public void updateActivityPlan()
    {
        this.setDownloading( true );
        ConnectionManager.updateActivityPlan();

    }

    public DataSet getSelectedDataSet()
    {
        return selectedDataSet;
    }

    public boolean isDownloading()
    {
        return downloading;
    }

    public void setDownloading( boolean downloading )
    {
        this.downloading = downloading;
    }

    public boolean isWaiting()
    {
        return waiting;
    }

    public void setWaiting( boolean waiting )
    {
        this.waiting = waiting;
    }

    private Command getVillageBackCmd()
    {
        if ( villageBackCmd == null )
        {
            villageBackCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return villageBackCmd;
    }

    private void updateDataSets()
    {
        // this.getWaitForm().deleteAll();
        // this.getWaitForm().setTitle( Text.UPDATE_DATASET() );
        // this.getWaitForm().append( Text.PLEASE_WAIT() );
        // this.switchDisplayable( null, this.getWaitForm() );

        // this.loadSettings();
        DataSetRecordStore dataSetRs = new DataSetRecordStore();
        try
        {
            Vector currentDataSets = dataSetRs.getAllDataSet();
            ConnectionManager.updateDataSets( currentDataSets );
        }
        catch ( RecordStoreException e )
        {
            e.printStackTrace();
        }
    }

    /**
     * Returns an initialized instance of exitCommand component.
     * 
     * @return the initialized component instance
     */
    public Command getExitCommand()
    {
        if ( exitCommand == null )
        {
            exitCommand = new Command( Text.EXIT(), Command.EXIT, 0 );
        }
        return exitCommand;
    }

    public List getMainMenuList()
    {
        if ( mainMenuList == null )
        {
            mainMenuList = new List( Text.MAIN_MENU(), Choice.IMPLICIT );
            mainMenuList.append( Text.ACTIVITY_PLAN(), null );
            mainMenuList.append( Text.COMPLETED_ACTIVITIES(), null );
            // mainMenuList.append( Text.UPDATE_ACTIVITY_PLAN(), null );
            mainMenuList.addCommand( getSettingCommand() );
            // mainMenuList.addCommand( getUpdateCommand() );
            mainMenuList.setCommandListener( this );
            mainMenuList.setFitPolicy( List.TEXT_WRAP_ON );
            mainMenuList.setSelectedFlags( new boolean[] { false, false, false, false } );
        }
        return mainMenuList;
    }

    private Command getSettingCommand()
    {
        if ( settingCommand == null )
        {
            settingCommand = new Command( Text.SETTINGS(), Command.SCREEN, 0 );
        }
        return settingCommand;
    }

    public Command getUpdateCommand()
    {
        if ( updateCommand == null )
        {
            updateCommand = new Command( Text.UPDATE_DATASET(), Command.SCREEN, 0 );
        }
        return updateCommand;
    }

    /**
     * Performs an action assigned to the selected list element in the
     * mainMenuList component.
     */
    public void mainMenuListAction()
    {
        String __selectedString = getMainMenuList().getString( getMainMenuList().getSelectedIndex() );
        if ( __selectedString != null )
        {
            if ( __selectedString.equals( Text.ACTIVITY_PLAN() ) )
            {
                if ( this.isDownloading() )
                {
                    this.getVillageForms().deleteAll();
                    this.getVillageForms().setTitle( "Activity Plan is updating..." );
                    switchDisplayable( null, getVillageForms() );
                }
                else
                {

                    this.displayVillageForms();
                }

            }
            else if ( __selectedString.equals( Text.COMPLETED_ACTIVITIES() ) )
            {
                this.checkExpiredActivities();
                this.currentActivity = false;
                switchDisplayable( null, getVillageForms() );

                this.prepairToLoadActivityPlan();
                ActivityRecordStore activityRecordStore = new ActivityRecordStore( this,
                    ActivityRecordStore.LOAD_COMPLETED_ACTIVITYPLAN );
                Thread thread = new Thread( activityRecordStore );
                thread.start();
            }
            else if ( __selectedString.equals( Text.UPDATE_ACT_PLAN() ) )
            {
                this.getWaitForm().deleteAll();
                this.getWaitForm().setTitle( Text.UPDATING() );
                this.getWaitForm().append( Text.PLEASE_WAIT() );
                this.switchDisplayable( null, this.getWaitForm() );

                this.loadSettings();
                ConnectionManager.updateActivityPlan();
            }
            else if ( __selectedString.equals( Text.UPDATE_ACTIVITY_PLAN() ) )
            {
                this.getWaitForm().deleteAll();
                this.getWaitForm().append( Text.PLEASE_WAIT() );
                this.switchDisplayable( null, this.getWaitForm() );
                this.initConnectionManager();
                ConnectionManager.updatePrograms();
            }
        }
    }

    public void displayVillageForms()
    {
        ActivityRecordStore activityRecordStore = new ActivityRecordStore( this,
            ActivityRecordStore.LOAD_CURRENT_ACTIVITYPLAN );
        Thread thread = new Thread( activityRecordStore );
        thread.start();
        this.getVillageForms().deleteAll();
        switchDisplayable( null, getVillageForms() );

    }

    private void checkExpiredActivities()
    {
        if ( this.serverCurrentDate != null )
        {
            ActivityRecordStore activityRecordStore = new ActivityRecordStore();
            activityRecordStore.deleteExpiredActivities( this.serverCurrentDate );
        }

    }

    // This flag is use to back from activityPlanList to whether villageForm or
    // to currentMainScreen
    public void setCurrentActivityFlag( boolean flag )
    {
        this.currentActivity = flag;
    }

    private void prepairToLoadActivityPlan()
    {
        getWaitForm().deleteAll();
        getWaitForm().setTitle( Text.LOADING_ACTIVITIES() );
        getWaitForm().append( Text.PLEASE_WAIT() );
        switchDisplayable( null, getWaitForm() );
    }

    public List getVillageForms()
    {
        if ( villageForm == null )
        {
            villageForm = new List( "", Choice.IMPLICIT );
            villageForm.addCommand( getVillageBackCmd() );
            villageForm.setCommandListener( this );
        }
        return villageForm;
    }

    /**
     * Fill villageForm with group attribute values
     * 
     * @return true: there are group attribute
     * @return false: there are not group attribute
     */
    public boolean fillVillageForm( List villageForm )
    {
        if ( attributes.size() > 0 )
        {
            getVillageForms().deleteAll();
            villageForm.setTitle( ((PatientAttribute) attributes.elementAt( 0 )).getName() );
            for ( int i = 0; i < attributes.size(); i++ )
            {
                villageForm.append( ((PatientAttribute) attributes.elementAt( i )).getValue(), null );
            }
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Returns an initialized instance of mnuListExtCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getMnuListExtCmd()
    {
        if ( mnuListExtCmd == null )
        {
            mnuListExtCmd = new Command( Text.EXIT(), Command.EXIT, 0 );
        }
        return mnuListExtCmd;
    }

    /**
     * Returns an initialized instance of mnuListDnldCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getMnuListDnldCmd()
    {
        if ( mnuListDnldCmd == null )
        {
            mnuListDnldCmd = new Command( Text.DOWNLOAD(), Command.SCREEN, 0 );
        }
        return mnuListDnldCmd;
    }

    /**
     * Returns an initialized instance of dsDnldListBakCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getDsDnldListBakCmd()
    {
        if ( dsDnldListBakCmd == null )
        {
            dsDnldListBakCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return dsDnldListBakCmd;
    }

    /**
     * Returns an initialized instance of frmDnldCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getFrmDnldCmd()
    {
        if ( frmDnldCmd == null )
        {
            frmDnldCmd = new Command( Text.DOWNLOAD(), Command.SCREEN, 0 );
        }
        return frmDnldCmd;
    }

    /**
     * Returns an initialized instance of actvPlnLstBakCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getActvPlnLstBakCmd()
    {
        if ( actvPlnLstBakCmd == null )
        {
            actvPlnLstBakCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return actvPlnLstBakCmd;
    }

    /**
     * Returns an initialized instance of settingsForm component.
     * 
     * @return the initialized component instance
     */
    public Form getSettingsForm()
    {
        if ( settingsForm == null )
        {
            settingsForm = new Form( Text.CONFIGURABLE_PARAMS(), new Item[] { getServerUrl(), getDhisUserName(),
                getDhisUserPass(), getLocale() } );
            settingsForm.addCommand( getSetngsSaveCmd() );
            settingsForm.addCommand( getSetngsBakCmd() );
            settingsForm.setCommandListener( this );
        }
        return settingsForm;
    }

    /**
     * Returns an initialized instance of stngsOkCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getStngsOkCmd()
    {
        if ( stngsOkCmd == null )
        {
            stngsOkCmd = new Command( Text.SAVE(), Command.OK, 0 );
        }
        return stngsOkCmd;
    }

    /**
     * Returns an initialized instance of setngsBakCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getSetngsBakCmd()
    {
        if ( setngsBakCmd == null )
        {
            setngsBakCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return setngsBakCmd;
    }

    /**
     * Returns an initialized instance of setngsSaveCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getSetngsSaveCmd()
    {
        if ( setngsSaveCmd == null )
        {
            setngsSaveCmd = new Command( Text.SAVE(), Command.SCREEN, 0 );
        }
        return setngsSaveCmd;
    }

    /**
     * Returns an initialized instance of url component.
     * 
     * @return the initialized component instance
     */
    public TextField getUrl()
    {
        if ( url == null )
        {
            url = new TextField( Text.URL(), DEFAULT_DHIS_URL, 64, TextField.URL );
        }
        return url;
    }

    /**
     * Returns an initialized instance of dhisUserPass component.
     * 
     * @return the initialized component instance
     */
    public TextField getDhisUserPass()
    {
        if ( dhisUserPass == null )
        {
            dhisUserPass = new TextField( Text.PASSWORD(), "", 32, TextField.ANY | TextField.PASSWORD );
        }
        return dhisUserPass;
    }

    /**
     * Returns an initialized instance of dataEntryForm component.
     * 
     * @return the initialized component instance
     */
    public Form getDataEntryForm()
    {
        if ( dataEntryForm == null )
        {
            dataEntryForm = new Form( Text.FORM(), new Item[] {} );
            dataEntryForm.addCommand( getDeFrmSndCmd() );
            dataEntryForm.addCommand( getDeFrmSavCmd() );
            dataEntryForm.addCommand( getDeFrmBakCmd() );
            dataEntryForm.setCommandListener( this );
        }
        return dataEntryForm;
    }

    /**
     * Returns an initialized instance of activityEntryForm component.
     * 
     */
    public Form getActivityEntryForm()
    {
        if ( activityEntryForm == null )
        {
            activityEntryForm = new Form( Text.FORM(), new Item[] {} );
            activityEntryForm.addCommand( getActivityFrmSndCmd() );
            activityEntryForm.addCommand( getActivityFrmSavCmd() );
            activityEntryForm.addCommand( getActivityFrmBakCmd() );
            activityEntryForm.setCommandListener( this );
        }
        return activityEntryForm;
    }

    /**
     * Returns an initialized instance of activityDetailCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getActivityDetailCmd()
    {
        if ( activityDetailCmd == null )
        {
            activityDetailCmd = new Command( Text.DETAILS(), Command.SCREEN, 0 );
        }
        return activityDetailCmd;
    }

    /**
     * Returns an initialized instance of activityDetailOkCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getActivityDetailOkCmd()
    {
        if ( activityDetailOkCmd == null )
        {
            activityDetailOkCmd = new Command( Text.OK(), Command.OK, 0 );
        }
        return activityDetailOkCmd;
    }

    /**
     * Returns an initialized instance of deFrmBakCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getDeFrmBakCmd()
    {
        if ( deFrmBakCmd == null )
        {
            deFrmBakCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return deFrmBakCmd;
    }

    /**
     * Returns an initialized instance of deFrmSndCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getDeFrmSndCmd()
    {
        if ( deFrmSndCmd == null )
        {
            deFrmSndCmd = new Command( Text.SEND(), Command.SCREEN, 0 );
        }
        return deFrmSndCmd;
    }

    /**
     * Returns an initialized instance of deFrmSavCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getDeFrmSavCmd()
    {
        if ( deFrmSavCmd == null )
        {
            deFrmSavCmd = new Command( Text.SAVE(), Command.SCREEN, 1 );
        }
        return deFrmSavCmd;
    }

    /**
     * Returns an initialized instance of activityFrmBakCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getActivityFrmBakCmd()
    {
        if ( activityFrmBakCmd == null )
        {
            activityFrmBakCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return activityFrmBakCmd;
    }

    /**
     * Returns an initialized instance of activityFrmSndCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getActivityFrmSndCmd()
    {
        if ( activityFrmSndCmd == null )
        {
            activityFrmSndCmd = new Command( Text.COMPLETE(), Command.SCREEN, 0 );
        }
        return activityFrmSndCmd;
    }

    /**
     * Returns an initialized instance of activityFrmSavCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getActivityFrmSavCmd()
    {
        if ( activityFrmSavCmd == null )
        {
            activityFrmSavCmd = new Command( Text.SAVE(), Command.SCREEN, 1 );
        }
        return activityFrmSavCmd;
    }

    /**
     * Returns an initialized instance of backCommand component.
     * 
     * @return the initialized component instance
     */
    public Command getBackCommand()
    {
        if ( backCommand == null )
        {
            backCommand = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return backCommand;
    }

    /**
     * Returns an initialized instance of screenCommand component.
     * 
     * @return the initialized component instance
     */
    public Command getScreenCommand()
    {
        if ( screenCommand == null )
        {
            screenCommand = new Command( Text.SAVE(), Command.SCREEN, 0 );
        }
        return screenCommand;
    }

    /**
     * Returns an initialized instance of loginForm component.
     * 
     * @return the initialized component instance
     */
    public Form getLoginForm()
    {
        if ( loginForm == null )
        {
            loginForm = new Form( Text.LOGIN(), new Item[] { getUserName(), getPassword(), getUrl() } );
            loginForm.addCommand( getLgnFrmExtCmd() );
            loginForm.addCommand( getLgnFrmLgnCmd() );
            loginForm.setCommandListener( this );
        }
        return loginForm;
    }

    public Form getPinForm()
    {
        if ( pinForm == null )
        {
            pinForm = new Form( Text.ENTER_PIN() );
            pinForm.append( this.getPinTextField() );
            pinForm.addCommand( this.getPinFormNextCmd() );
            pinForm.addCommand( this.getExitCommand() );
            pinForm.setCommandListener( this );
        }
        else
        {
            getPinTextField().setString( "" );
            if ( pinForm.size() > 2 )
            {
                pinForm.delete( 1 );
            }
        }
        return pinForm;
    }

    public TextField getPinTextField()
    {
        if ( pinTextField == null )
        {
            pinTextField = new TextField( Text.PIN(), "", 4, TextField.NUMERIC | TextField.PASSWORD );

        }
        return pinTextField;
    }

    private Command getPinFormNextCmd()
    {
        if ( pinFormNextCmd == null )
        {
            pinFormNextCmd = new Command( Text.NEXT(), Command.SCREEN, 0 );
        }
        return pinFormNextCmd;
    }

    private Command getPinFormReinitCmd()
    {
        if ( pinFormReinitCmd == null )
        {
            pinFormReinitCmd = new Command( Text.REINIT(), Command.SCREEN, 1 );
        }
        return pinFormReinitCmd;
    }

    /**
     * Returns an initialized instance of userName component.
     * 
     * @return the initialized component instance
     */
    public TextField getUserName()
    {
        if ( userName == null )
        {
            userName = new TextField( Text.USERNAME(), "", 32, TextField.ANY | TextField.SENSITIVE );
        }
        return userName;
    }

    /**
     * Returns an initialized instance of password component.
     * 
     * @return the initialized component instance
     */
    public TextField getPassword()
    {
        if ( password == null )
        {
            password = new TextField( Text.PASSWORD(), "", 32, TextField.ANY | TextField.PASSWORD );
        }
        return password;
    }

    public TextField getServerUrl()
    {
        if ( serverURL == null )
        {
            serverURL = new TextField( Text.URL(), "", 64, TextField.URL );
        }
        return serverURL;
    }

    /**
     * Returns an initialized instance of lgnFrmExtCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getLgnFrmExtCmd()
    {
        if ( lgnFrmExtCmd == null )
        {
            lgnFrmExtCmd = new Command( Text.EXIT(), Command.EXIT, 0 );
        }
        return lgnFrmExtCmd;
    }

    /**
     * Returns an initialized instance of okCommand component.
     * 
     * @return the initialized component instance
     */
    public Command getOkCommand()
    {
        if ( okCommand == null )
        {
            okCommand = new Command( Text.OK(), Command.OK, 0 );
        }
        return okCommand;
    }

    /**
     * Returns an initialized instance of lgnFrmLgnCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getLgnFrmLgnCmd()
    {
        if ( lgnFrmLgnCmd == null )
        {
            lgnFrmLgnCmd = new Command( Text.LOGIN(), Command.SCREEN, 0 );
        }
        return lgnFrmLgnCmd;
    }

    /**
     * Returns an initialized instance of logo component.
     * 
     * @return the initialized component instance
     */
    public Image getLogo()
    {
        if ( logo == null )
        {
            try
            {
                logo = Image.createImage( LOGO_IMAGE );
            }
            catch ( java.io.IOException e )
            {
                e.printStackTrace();
            }
        }
        return logo;
    }

    /**
     * Returns an initialized instance of activityDetailForm component.
     * 
     * @return the initialized component instance
     */
    public Form getActivityDetailForm()
    {
        if ( activityDetailForm == null )
        {
            activityDetailForm = new Form( Text.DETAILS() );
            activityDetailForm.addCommand( getActivityDetailOkCmd() );
            activityDetailForm.setCommandListener( this );
        }
        return activityDetailForm;
    }

    /**
     * Returns an initialized instance of activityPlanList component.
     * 
     * @return the initialized component instance
     */
    public List getActivityPlanList()
    {
        if ( activityPlanList == null )
        {
            activityPlanList = new List( Text.SELECT_ACTIVITY(), Choice.IMPLICIT );
            activityPlanList.setFitPolicy( Choice.TEXT_WRAP_ON );
            activityPlanList.addCommand( getActvPlnLstBakCmd() );
            // activityPlanList.addCommand( getSettingCommand() );
            activityPlanList.setCommandListener( this );
        }
        return activityPlanList;
    }

    /**
     * Returns an initialized instance of dataSetDisplayList component.
     * 
     * @return the initialized component instance
     */
    public List getDataSetDisplayList()
    {
        if ( dataSetDisplayList == null )
        {
            dataSetDisplayList = new List( Text.SELECT_FORM(), Choice.IMPLICIT );
            dataSetDisplayList.setFitPolicy( List.TEXT_WRAP_ON );
            dataSetDisplayList.addCommand( getSettingCommand() );
            // dataSetDisplayList.addCommand( getUpdateCommand() );
            dataSetDisplayList.setCommandListener( this );
        }
        return dataSetDisplayList;
    }

    /**
     * Performs an action assigned to the selected list element in the
     * dataSetDisplayList component.
     */
    public void dataSetDisplayListAction()
    {

        Model model = (Model) dataSetsVector.elementAt( ((List) getDataSetDisplayList()).getSelectedIndex() );
        selectedDataSet = fetchDataSet( model.getId() );
        switchDisplayable( null, getPeriodForm() );
        getPeriodForm().removeCommand( getSelectDailyPeriodCmd() );
        getPeriodForm().deleteAll();

        if ( selectedDataSet.getPeriodType().equals( "Daily" ) )
        {
            getDailyPeriodDateField();
            getPeriodForm().append( getDailyPeriodDateField() );
            getPeriodForm().addCommand( getSelectDailyPeriodCmd() );

        }
        else
        {
            getPeriodForm().append( getPeriodChoice() );
            Vector periods = new Vector();

            if ( selectedDataSet.getPeriodType().equals( "Monthly" ) )
            {
                periods = Period.generateMonthlyPeriods();
            }

            else if ( selectedDataSet.getPeriodType().equals( "Yearly" ) )
                periods = Period.generateYearlyPeriods();
            else if ( selectedDataSet.getPeriodType().equals( "Weekly" ) )
                periods = Period.generateWeeklyPeriods();
            else if ( selectedDataSet.getPeriodType().equals( "Quarterly" ) )
                periods = Period.generateQuaterlyPeriods();

            getPeriodChoice().deleteAll();

            for ( int i = 0; i < periods.size(); i++ )
            {
                String period = (String) periods.elementAt( i );
                short state = DataSetValueRecordStore.getDataSetState( selectedDataSet, period );
                if ( state == DataSetValueRecordStore.STATE_SAVED )
                {
                    period = period + " * " + Text.SAVED();
                }
                else if ( state == DataSetValueRecordStore.STATE_COMPLETED )
                {
                    period = period + " * " + Text.COMPLETED();
                }
                getPeriodChoice().append( period, null );
                period = null;
            }
            System.gc();
        }
    }

    /**
     * Returns an initialized instance of dsLstBakCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getDsLstBakCmd()
    {
        if ( dsLstBakCmd == null )
        {
            dsLstBakCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return dsLstBakCmd;
    }

    /**
     * Returns an initialized instance of servicesList component.
     * 
     * @return the initialized component instance
     */
    public List getServicesList()
    {
        if ( servicesList == null )
        {
            servicesList = new List( Text.SERVICES(), Choice.IMPLICIT );
            servicesList.append( Text.FACILITY_REPORTING(), null );
            servicesList.append( Text.ACTIVITY_REPORTING(), null );
            servicesList.addCommand( getSrvcsExitCmd() );
            servicesList.addCommand( getSettingCommand() );
            servicesList.setCommandListener( this );
            servicesList.setSelectedFlags( new boolean[] { false, false } );
        }
        return servicesList;
    }

    /**
     * Performs an action assigned to the selected list element in the
     * servicesList component.
     */
    public void servicesListAction()
    {
        String __selectedString = getServicesList().getString( getServicesList().getSelectedIndex() );
        if ( __selectedString != null )
        {
            if ( __selectedString.equals( Text.FACILITY_REPORTING() ) )
            {

                loadDataSets( getDataSetDisplayList() );
                switchDisplayable( null, getDataSetDisplayList() );

            }
            else if ( __selectedString.equals( Text.ACTIVITY_REPORTING() ) )
            {
                this.switchDisplayable( null, getMainMenuList() );
            }
        }
    }

    /**
     * Performs an action assigned to the selected list element in the
     * servicesList component.
     */
    public void activityListAction()
    {
        String name = getActivityPlanList().getString( getActivityPlanList().getSelectedIndex() );
        String duedate = (Period.split( name, '\n' ))[1].trim();
        name = (name.indexOf( "*" ) == -1) ? name.substring( 0, name.indexOf( "\n" ) ) : name.substring(
            name.indexOf( "*" ) + 1, name.indexOf( "\n" ) );
        for ( int i = 0; i < activitiesVector.size(); i++ )
        {
            Activity activity = (Activity) activitiesVector.elementAt( i );
            if ( activity.getBeneficiary().getFullName().equalsIgnoreCase( name.trim() )
                && duedate.equals( Period.formatDate( activity.getDueDate() ) ) )
            {
                selectedActivity = activity;
                break;
            }
        }
        activityDetailsAction();
        switchDisplayable( null, getActivityDetailForm() );
        System.gc();
    }

    // Display Current Activity
    public void displayCurrentActivity( ProgramStage programStage, Form form )
    {

        if ( selectedActivity == null || programStage == null )
        {
            form.deleteAll();
            form.append( Text.FORM_UNAVAILABLE() );
        }
        else
        {
            ActivityValueRecordStore activityValueRecordStore = new ActivityValueRecordStore();
            ActivityValue activityValue = activityValueRecordStore.load( selectedActivity.getTask()
                .getProgStageInstId() );
            form.deleteAll();
            form.setTitle( programStage.getName() );
            Vector des = programStage.getDataElements();
            selectedProgramStage = programStage;

            // If ActivityValue is not null
            if ( activityValue != null )
            {
                // Get DataValue for each DataElement of the form
                for ( int i = 0; i < des.size(); i++ )
                {
                    DataElement de = (DataElement) des.elementAt( i );
                    // Get DataValue from Hashtable
                    DataValue dataValue = (DataValue) activityValue.getDataValues().get( String.valueOf( de.getId() ) );
                    // Put value into field

                    // Check compulsory
                    String fieldTitle = de.isCompulsory() ? de.getName() + "*" : de.getName();

                    if ( de.getType().equals( "date" ) )
                    {
                        //Edit
                        if( dataValue != null && !dataValue.getVal().equalsIgnoreCase( "" ) ){
                            if( serverCurrentDate != null ){
                                ChoiceGroup dateCbx = new ChoiceGroup( fieldTitle, ChoiceGroup.EXCLUSIVE );
                                String serverDateStr = Period.formatDate( serverCurrentDate );
                                if(serverDateStr.equals( dataValue.getVal() )){
                                    dateCbx.append( serverDateStr , null );
                                }else{
                                    dateCbx.append( serverDateStr , null );
                                    dateCbx.append( dataValue.getVal(), null );
                                    dateCbx.setSelectedIndex( 1, true );
                                }
                                form.append( dateCbx );
                                dataElements.put( String.valueOf( de.getId() ), dateCbx );
                                
                                DateField dateField = new DateField( fieldTitle, DateField.DATE );
                                dateField.setDate( Period.stringToDate( dataValue.getVal() ) );                                
                                dateFieldValues.put( String.valueOf( de.getId() ), dateField );
                                
                            }else{
                                DateField dateField = new DateField( fieldTitle, DateField.DATE );
                                dateField.setDate( Period.stringToDate( dataValue.getVal() ) );
                                form.append( dateField );
                                dataElements.put( String.valueOf( de.getId() ), dateField );
                            }
                        }else{
                            if ( serverCurrentDate != null ){
                                ChoiceGroup dateCbx = new ChoiceGroup( fieldTitle, ChoiceGroup.MULTIPLE );
                                dateCbx.append( Period.formatDate( serverCurrentDate ), null );
                                form.append( dateCbx );
                                dataElements.put( String.valueOf( de.getId() ), dateCbx );
                            }else{
                                DateField dateField = new DateField( fieldTitle, DateField.DATE );
                                form.append( dateField );
                                dataElements.put( String.valueOf( de.getId() ), dateField );
                            }
                        }
                    }
                    else if ( de.getType().equals( "int" ) )
                    {
                        TextField intField = new TextField( fieldTitle, "", TEXT_NUMERIC, TextField.NUMERIC );
                        if ( dataValue != null )
                        {
                            intField.setString( dataValue.getVal() );
                        }
                        form.append( intField );
                        dataElements.put( String.valueOf( de.getId() ), intField );
                    }
                    else if ( de.getType().equals( "bool" ) )
                    {
                        choiceGroup = new ChoiceGroup( fieldTitle, Choice.POPUP );
                        choiceGroup.append( Text.SELECT_OPTION(), null );
                        choiceGroup.append( Text.YES(), null );
                        choiceGroup.append( Text.NO(), null );
                        choiceGroup.setFitPolicy( Choice.TEXT_WRAP_ON );
                        if ( dataValue != null )
                        {
                            if ( dataValue.getVal().equals( "" ) )
                            {
                                choiceGroup.setSelectedIndex( 0, true );
                            }
                            else if ( dataValue.getVal().equals( "true" ) )
                            {
                                choiceGroup.setSelectedIndex( 1, true );
                            }
                            else
                            {
                                choiceGroup.setSelectedIndex( 2, true );
                            }
                        }
                        form.append( choiceGroup );
                        dataElements.put( String.valueOf( de.getId() ), choiceGroup );

                    }
                    else if ( de.getType().equals( "string" ) )
                    {
                        // Case DataElement has default CategoryOptCombo
                        if ( de.getCategoryOptionCombos().getModels().size() <= 1 )
                        {
                            TextField txtField = new TextField( fieldTitle, "", 32, TextField.ANY );
                            if ( dataValue != null )
                            {
                                txtField.setString( dataValue.getVal() );
                            }
                            form.append( txtField );
                            dataElements.put( String.valueOf( de.getId() ), txtField );
                        }
                        // Case DataElement has more than one CategoryOptCombo
                        else
                        {
                            ChoiceGroup choiceGroup = new ChoiceGroup( fieldTitle, ChoiceGroup.POPUP );
                            Vector categoryOptCombos = de.getCategoryOptionCombos().getModels();
                            choiceGroup.append( Text.SELECT_OPTION(), null );
                            for ( int j = 0; j < categoryOptCombos.size(); j++ )
                            {
                                Model cateOptCombo = (Model) categoryOptCombos.elementAt( j );
                                choiceGroup.append( cateOptCombo.getName(), null );
                                if ( dataValue != null && !dataValue.getVal().equals( "" ) )
                                {
                                    if ( cateOptCombo.getId() == dataValue.getCategoryOptComboID() )
                                    {
                                        choiceGroup.setSelectedIndex( j + 1, true );
                                    }
                                }
                                cateOptCombo = null;
                            }
                            form.append( choiceGroup );
                            dataElements.put( String.valueOf( de.getId() ), choiceGroup );
                            categoryOptCombos = null;
                        }
                    }
                    de = null;
                    System.gc();
                }
            }
            else
            {
                for ( int i = 0; i < des.size(); i++ )
                {
                    DataElement de = (DataElement) des.elementAt( i );
                    // Check compulsory
                    String fieldTitle = de.isCompulsory() ? de.getName() + "*" : de.getName();

                    if ( de.getType().equals( "date" ) )
                    {
                        //New Input
                        if ( serverCurrentDate != null ){
                            ChoiceGroup dateCbx = new ChoiceGroup( fieldTitle, ChoiceGroup.MULTIPLE );
                            dateCbx.append( Period.formatDate( serverCurrentDate ), null );
                            form.append( dateCbx );
                            dataElements.put( String.valueOf( de.getId() ), dateCbx );
                        }else{
                            DateField dateField = new DateField( fieldTitle, DateField.DATE );
                            form.append( dateField );
                            dataElements.put( String.valueOf( de.getId() ), dateField );
                        }
                    }
                    else if ( de.getType().equals( "int" ) )
                    {
                        TextField intField = new TextField( fieldTitle, "", TEXT_NUMERIC, TextField.NUMERIC );
                        form.append( intField );
                        dataElements.put( String.valueOf( de.getId() ), intField );

                    }
                    else if ( de.getType().equals( "bool" ) )
                    {
                        choiceGroup = new ChoiceGroup( fieldTitle, Choice.POPUP );
                        choiceGroup.append( Text.SELECT_OPTION(), null );
                        choiceGroup.append( Text.YES(), null );
                        choiceGroup.append( Text.NO(), null );
                        choiceGroup.setFitPolicy( Choice.TEXT_WRAP_ON );
                        form.append( choiceGroup );
                        dataElements.put( String.valueOf( de.getId() ), choiceGroup );
                    }
                    else if ( de.getType().equals( "string" ) )
                    {
                        if ( de.getCategoryOptionCombos().getModels().size() <= 1 )
                        {
                            TextField txtField = new TextField( fieldTitle, "", 32, TextField.ANY );
                            form.append( txtField );
                            dataElements.put( String.valueOf( de.getId() ), txtField );
                        }
                        else
                        {
                            ChoiceGroup choiceGroup = new ChoiceGroup( fieldTitle, ChoiceGroup.POPUP );
                            choiceGroup.append( Text.SELECT_OPTION(), null );
                            Vector categoryOptCombos = de.getCategoryOptionCombos().getModels();
                            for ( int j = 0; j < categoryOptCombos.size(); j++ )
                            {
                                Model categoryOptCombo = (Model) categoryOptCombos.elementAt( j );
                                choiceGroup.append( categoryOptCombo.getName(), null );
                            }
                            form.append( choiceGroup );
                            dataElements.put( String.valueOf( de.getId() ), choiceGroup );
                        }
                    }
                }
            }
        }
        switchDisplayable( null, form );
    }

    // Display Completed Activity
    private void displayCompletedActivity( ProgramStage programStage, Form form )
    {
        ActivityValueRecordStore activityValueRecordStore = new ActivityValueRecordStore();
        ActivityValue activityValue = activityValueRecordStore.load( selectedActivity.getTask().getProgStageInstId() );
        form.deleteAll();
        form.setTitle( programStage.getName() );
        form.addCommand( getUnEditableBckCmd() );
        Vector des = programStage.getDataElements();
        selectedProgramStage = programStage;
        form.setTitle( programStage.getName() );
        if ( activityValue != null )
        {
            // Get DataValue for each DataElement of the form
            for ( int i = 0; i < des.size(); i++ )
            {
                DataElement de = (DataElement) des.elementAt( i );
                DataValue dataValue = (DataValue) activityValue.getDataValues().get( String.valueOf( de.getId() ) );

                // Put value into field
                TextField txtField = new TextField( de.getName(), "", TEXT_STRING, TextField.ANY | TextField.UNEDITABLE );
                if ( de.getType().equals( "bool" ) && dataValue != null )
                {
                    if ( dataValue.getVal().equalsIgnoreCase( "true" ) )
                    {
                        txtField.setString( Text.YES() );
                    }
                    else if ( dataValue.getVal().equalsIgnoreCase( "false" ) )
                    {
                        txtField.setString( Text.NO() );
                    }
                }
                else
                {
                    if ( dataValue != null )
                    {
                        if ( de.getCategoryOptionCombos().getModels().size() <= 1 )
                        {
                            txtField.setString( dataValue.getVal() );
                        }
                        else
                        {
                            Vector cateCombos = de.getCategoryOptionCombos().getModels();
                            if ( !dataValue.getVal().equals( "" ) )
                            {
                                for ( int j = 0; j < cateCombos.size(); j++ )
                                {
                                    Model cateCombo = (Model) cateCombos.elementAt( j );
                                    if ( cateCombo.getId() == dataValue.getCategoryOptComboID() )
                                    {
                                        txtField.setString( cateCombo.getName() );
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
                form.append( txtField );
                dataValue = null;
                de = null;
                System.gc();
            }
        }
        switchDisplayable( null, form );
    }

    /**
     * Performs an action assigned to the selected list element in the
     * activityPlanList component.
     */
    public void activityDetailsAction()
    {
        getActivityDetailForm().deleteAll();

        ModelRecordStore modelRecordStore = null;

        Program program = null;

        try
        {
            selectedProgramStage = null;
            modelRecordStore = new ModelRecordStore( ModelRecordStore.PROGRAM_DB );
            byte[] rec = modelRecordStore.getRecord( selectedActivity.getTask().getProgramId() );

            if ( rec != null )
            {
                program = new Program();
                SerializationUtil.deSerialize( program, rec );

                for ( int i = 0; i < program.getProgramStages().size(); i++ )
                {
                    if ( selectedActivity.getTask().getProgStageId() == ((ProgramStage) program.getProgramStages()
                        .elementAt( i )).getId() )
                    {
                        selectedProgramStage = (ProgramStage) program.getProgramStages().elementAt( i );
                        break;
                    }
                }
                StringBuffer buffer = new StringBuffer();
                getActivityDetailForm().setTitle(
                    selectedProgramStage.getName() + " - " + Period.formatDate( selectedActivity.getDueDate() ) );

                buffer.append( "Name:  " );
                buffer.append( selectedActivity.getBeneficiary().getFullName() );
                buffer.append( "\n" );
                buffer.append( "Age: " );
                buffer.append( selectedActivity.getBeneficiary().getAge() );
                buffer.append( "\n" );
                buffer.append( generatePatientIdentifiers( selectedActivity.getBeneficiary().getPatientIdentifier() ) );
                buffer.append( generateStaticPatientAttributes( selectedActivity.getBeneficiary() ) );
                buffer.append( generatePatientAtts( selectedActivity.getBeneficiary().getAttsValues() ) );

                getActivityDetailForm().append( buffer.toString() );
                System.gc();
            }
            else
            {
                getActivityDetailForm().deleteAll();
                getActivityDetailForm().append( "There are no details" );

            }

        }
        catch ( RecordStoreException e )
        {
            System.out.println( e.getMessage() );
            e.printStackTrace();
        }
        catch ( IOException e )
        {
            System.out.println( e.getMessage() );
            e.printStackTrace();
        }
        finally
        {
            System.gc();
        }

    }

    private String generatePatientIdentifiers( Vector identifiers )
    {
        if ( identifiers != null )
        {
            String identifierStr = "";
            for ( int i = 0; i < identifiers.size(); i++ )
            {
                identifierStr += ((PatientIdentifier) identifiers.elementAt( i )).getIdentifierType() + ": "
                    + ((PatientIdentifier) identifiers.elementAt( i )).getIdentifier() + "\n";
            }
            return identifierStr;
        }
        else
        {
            return "";
        }

    }

    private String generateStaticPatientAttributes( Beneficiary beneficiary )
    {
        String staticAtts = "";

        if ( beneficiary.getGender() != null )
        {
            if ( beneficiary.getGender().equals( "F" ) )
            {
                staticAtts += "Gender: Female" + "\n";
            }
            else
            {
                staticAtts += "Gender: Male" + "\n";
            }

        }
        if ( beneficiary.getBirthDate() != null )
        {
            String dob = beneficiary.getBirthDate().toString();
            staticAtts += "DOB: " + dob.substring( 4, 10 ) + " " + dob.substring( 23 );
        }
        if ( beneficiary.getDobType() != null )
        {
            if ( beneficiary.getDobType().charValue() == 'V' )
            {
                staticAtts += " (Verified)";
            }
            else if ( beneficiary.getDobType().charValue() == 'D' )
            {
                staticAtts += " (Defined)";
            }
            else
            {
                staticAtts += " (Approximated)";
            }
        }
        if ( beneficiary.getBloodGroup() != null )
        {
            staticAtts += "\nBlood Group: " + beneficiary.getBloodGroup() + "\n";
        }
        if ( beneficiary.getRegistrationDate() != null )
        {
            String regDate = beneficiary.getRegistrationDate().toString();
            staticAtts += "Registration Date: " + regDate.substring( 4, 10 ) + " " + regDate.substring( 23 ) + "\n";
        }

        return staticAtts;
    }

    private String generatePatientAtts( Vector atts )
    {
        String infos = "";
        for ( int i = 0; i < atts.size(); i++ )
        {
            infos += "\n" + atts.elementAt( i ) + "\n";
        }
        return infos;
    }

    public Command getSrvcsExitCmd()
    {
        if ( srvcsExitCmd == null )
        {
            srvcsExitCmd = new Command( Text.EXIT(), Command.EXIT, 0 );
        }
        return srvcsExitCmd;
    }

    public TextField getDhisUserName()
    {
        if ( dhisUserName == null )
        {
            dhisUserName = new TextField( Text.USERNAME(), null, 32, TextField.ANY );
        }
        return dhisUserName;
    }

    /**
     * Returns an initialized instance of locale component.
     * 
     * @return the initialized component instance
     */
    public TextField getLocale()
    {
        if ( locale == null )
        {
            locale = new TextField( "Language Locale", null, 32, TextField.ANY );
        }
        return locale;
    }

    /**
     * Returns an initialized instance of periodForm component.
     * 
     * @return the initialized component instance
     */
    public Form getPeriodForm()
    {
        if ( periodForm == null )
        {
            periodForm = new Form( Text.SELECT_PERIOD(), new Item[] { getPeriodChoice() } );
            periodForm.addCommand( getPeriodNxtCmd() );
            periodForm.addCommand( getPeriodBakCmd() );
            periodForm.setCommandListener( this );
        }
        return periodForm;
    }

    /**
     * Returns an initialized instance of periodBakCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getPeriodBakCmd()
    {
        if ( periodBakCmd == null )
        {
            periodBakCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return periodBakCmd;
    }

    /**
     * Returns an initialized instance of periodNxtCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getPeriodNxtCmd()
    {
        if ( periodNxtCmd == null )
        {
            periodNxtCmd = new Command( Text.NEXT(), Command.SCREEN, 0 );
        }
        return periodNxtCmd;
    }

    /**
     * Returns an initialized instance of periodChoice component.
     * 
     * @return the initialized component instance
     */
    public ChoiceGroup getPeriodChoice()
    {
        if ( periodChoice == null )
        {
            periodChoice = new ChoiceGroup( Text.PERIOD(), Choice.POPUP );
            periodChoice.setFitPolicy( Choice.TEXT_WRAP_DEFAULT );
        }
        return periodChoice;
    }

    /**
     * Returns an initialized instance of waitForm component.
     * 
     * @return the initialized component instance
     */
    public Form getWaitForm()
    {
        if ( waitForm == null )
        {
            waitForm = new Form( Text.PLEASE_WAIT() );
        }
        return waitForm;
    }

    /**
     * Returns an initialized instance of deleteProgList component.
     * 
     * @return the initialized component instance
     */
    public List getDeleteProgList()
    {
        if ( deleteProgList == null )
        {
            deleteProgList = new List( Text.PLEASE_SELECT(), Choice.IMPLICIT );
            deleteProgList.addCommand( getDeleteBakCmd() );
            deleteProgList.setCommandListener( this );
        }
        return deleteProgList;
    }

    /**
     * Returns an initialized instance of deleteList component.
     * 
     * @return the initialized component instance
     */
    public List getDeleteList()
    {
        if ( deleteList == null )
        {
            deleteList = new List( Text.PLEASE_SELECT(), Choice.IMPLICIT );
            deleteList.addCommand( getDeleteBakCmd() );
            deleteList.setCommandListener( this );
        }
        if ( deleteList.size() > 0 )
        {
            deleteList.addCommand( getDsDeleteCmd() );
        }
        else
        {
            deleteList.removeCommand( getDsDeleteCmd() );
        }
        return deleteList;
    }

    /**
     * Performs an action assigned to the selected list element in the
     * deleteList component.
     */
    public void deleteListAction()
    {

    }

    /**
     * Returns an initialized instance of deleteBakCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getDeleteBakCmd()
    {
        if ( deleteBakCmd == null )
        {
            deleteBakCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return deleteBakCmd;
    }

    /**
     * Returns an initialized instance of dsDeleteCmd component.
     * 
     * @return the initialized component instance
     */
    public Command getDsDeleteCmd()
    {
        if ( dsDeleteCmd == null )
        {
            dsDeleteCmd = new Command( Text.DELETE(), Command.SCREEN, 0 );
        }
        return dsDeleteCmd;
    }

    /**
     * Returns a display instance.
     * 
     * @return the display instance.
     */
    public Display getDisplay()
    {
        return Display.getDisplay( this );
    }

    /**
     * Exits MIDlet.
     */
    public void exitMIDlet()
    {
        switchDisplayable( null, null );
        destroyApp( true );
        notifyDestroyed();
    }

    /**
     * Called when MIDlet is started. Checks whether the MIDlet have been
     * already started and initialize/starts or resumes the MIDlet.
     */
    public void startApp()
    {
        TextMapping textMapping = null;
        try
        {
            SettingsRecordStore settingRecordStore = new SettingsRecordStore( SettingsRecordStore.SETTINGS_DB );
            String locale = settingRecordStore.get( "locale" );
            if ( !DEFAULT_LOCALE.equals( locale ) && !locale.equals( "" ) )
            {
                textMapping = TextMappingLoader.load( locale );
                Text.setTextMapping( textMapping );
            }
        }
        catch ( Exception e )
        {
            handleError( e );
        }

        if ( midletPaused )
        {
            resumeMIDlet();
        }
        else
        {
            startMIDlet();
        }
        midletPaused = false;
    }

    private void handleError( Exception e )
    {
        System.out.println( e.getMessage() );
        e.printStackTrace();
        Alert alert = Alerts.getInfoAlert( Text.ERROR(), e.getMessage() );
        alert.setTimeout( 3500 );
        Display.getDisplay( this ).setCurrent( alert );
        try
        {
            Thread.sleep( 3500 );
        }
        catch ( InterruptedException e1 )
        {
            e1.printStackTrace();
        }
    }

    /**
     * Called when MIDlet is paused.
     */
    public void pauseApp()
    {
        midletPaused = true;
    }

    /**
     * Called to signal the MIDlet to terminate.
     * 
     * @param unconditional if true, then the MIDlet has to be unconditionally
     *        terminated and all resources has to be released.
     */
    public void destroyApp( boolean unconditional )
    {
    }

    // Login Method
    private void login()
    {
        if ( getUserName().getString() != null && getPassword().getString() != null )
        {
            if ( getUserName().getString().trim().length() != 0 && getPassword().getString().trim().length() != 0 )
            {
                initConnectionManager();
                ConnectionManager.login();
            }
        }
    }

    private void initConnectionManager()
    {
        String url = getUrl().getString();
        url += (url.endsWith( "/" ) ? "" : "/") + MOBILE_PATH;
        ConnectionManager.init( this, url, getUserName().getString(), getPassword().getString(), getLocale()
            .getString(), orgUnit );
    }

    // Save setting to Record Store
    private void saveSettings()
    {
        SettingsRecordStore settingsRecord;
        try
        {

            settingsRecord = new SettingsRecordStore( SettingsRecordStore.SETTINGS_DB );
            settingsRecord.put( "url", getServerUrl().getString() );
            settingsRecord.put( "username", getDhisUserName().getString() );
            settingsRecord.put( "password", getDhisUserPass().getString() );
            settingsRecord.put( "locale", getLocale().getString() );
            settingsRecord.save();

        }
        catch ( RecordStoreException rse )
        {
        }
    }

    /**
     * Display the list of activities
     * 
     * @param activities Vector of activities to display
     * @param condition group activities?
     */
    public void loadActivityPlan( Vector activities, boolean condition )
    {
        activitiesVector.removeAllElements();
        attributes.removeAllElements();
        Hashtable temp = new Hashtable();
        activitiesVector = activities;

        getActivityPlanList().deleteAll();

        for ( int i = 0; i < activities.size(); i++ )
        {
            Activity activity = (Activity) activities.elementAt( i );

            if ( condition )
            {
                // Extract group factor attribute of beneficiary from activity
                PatientAttribute patientAttribute = activity.getBeneficiary().getGroupByAttribute();

                if ( patientAttribute != null && temp.get( patientAttribute.getValue().trim() ) == null )
                {
                    attributes.addElement( patientAttribute );
                    temp.put( patientAttribute.getValue().trim(), patientAttribute );
                }
                // End

                // Sort Group-Factor Attribute List
                if ( attributes.size() > 1 )
                {
                    sortAttributeList( attributes );
                }
            }

        }
        temp.clear();
        temp = null;
        System.gc();
    }

    private void sortAttributeList( Vector attributes )
    {
        PatientAttribute temp;
        int size = attributes.size();
        int i = 0;
        while ( i < size )
        {
            for ( int j = 0; j < size - 1; j++ )
            {
                if ( ((PatientAttribute) attributes.elementAt( j )).compareTo( (PatientAttribute) attributes
                    .elementAt( j + 1 ) ) > 0 )
                {
                    temp = new PatientAttribute( ((PatientAttribute) attributes.elementAt( j )).getName(),
                        ((PatientAttribute) attributes.elementAt( j )).getValue() );
                    attributes.setElementAt( attributes.elementAt( j + 1 ), j );
                    attributes.setElementAt( new PatientAttribute( temp.getName(), temp.getValue() ), j + 1 );
                    temp = null;
                }
                i++;
            }
        }
    }

    // Fill activityPlanList
    public void fillActivityPlanList( PatientAttribute groupFactor, boolean condition )
    {
        if ( !condition )
        {
            for ( int i = 0; i < activitiesVector.size(); i++ )
            {
                Activity activity = (Activity) activitiesVector.elementAt( i );

                String displayName = activity.getBeneficiary().getFullName() + "\n"
                    + Period.formatDate( activity.getDueDate() );

                if ( activity.isLate() && activity.getTask().isComplete() == false )
                {
                    displayName = "*" + displayName;
                }

                getActivityPlanList().insert( i, displayName, null );
            }
        }
        else
        {
            getActivityPlanList().deleteAll();
            for ( int i = 0; i < activitiesVector.size(); i++ )
            {
                Activity activity = (Activity) activitiesVector.elementAt( i );

                if ( activity.getBeneficiary().getGroupByAttribute().getValue().equals( groupFactor.getValue() ) )
                {
                    String displayName = activity.getBeneficiary().getFullName() + "\n"
                        + Period.formatDate( activity.getDueDate() );
                    if ( activity.isLate() && activity.getTask().isComplete() == false )
                    {
                        displayName = "*" + displayName;
                    }

                    getActivityPlanList().append( displayName, null );
                }
            }
        }
    }

    // Refresh the list of Activity
    public void refreshActivityPlan()
    {
        for ( int i = 0; i < activitiesVector.size(); i++ )
        {
            Activity activity = (Activity) activitiesVector.elementAt( i );
            if ( selectedActivity.getTask().getProgStageInstId() == activity.getTask().getProgStageInstId() )
            {
                activitiesVector.removeElementAt( i );
                return;
            }
            activity = null;
        }
    }

    // Load DataSet from Record Store to dataSetsVector
    public void loadDataSets( List list )
    {
        dataSetsVector.removeAllElements();
        ModelRecordStore modelRecordStore = null;
        Vector dataSets = null;
        try
        {
            modelRecordStore = new ModelRecordStore( ModelRecordStore.DATASET_DB );
            dataSets = modelRecordStore.getAllRecord();
        }
        catch ( RecordStoreException rse )
        {
        }

        list.deleteAll();

        for ( int i = 0; i < dataSets.size(); i++ )
        {
            Model ds = (Model) dataSets.elementAt( i );
            list.insert( i, ds.getName(), null );
            dataSetsVector.addElement( ds );
        }
    }

    // Load settings from record store to settings fields
    private void loadSettings()
    {
        SettingsRecordStore settingsRecord;

        try
        {
            settingsRecord = new SettingsRecordStore( SettingsRecordStore.SETTINGS_DB );

            String localeSetting = "";
            localeSetting = settingsRecord.get( "locale" );

            if ( localeSetting.trim().length() == 0 )
            {
                localeSetting = System.getProperty( "microedition.locale" );
                if ( localeSetting == null )
                {
                    localeSetting = "en-US";
                }
            }

            // Item of Login screen
            getUrl().setString( settingsRecord.get( "url" ) );
            getPassword().setString( settingsRecord.get( "password" ) );
            getUserName().setString( settingsRecord.get( "username" ) );

            // Item of Setting screen
            getServerUrl().setString( settingsRecord.get( "url" ) );
            getDhisUserName().setString( settingsRecord.get( "username" ) );
            getDhisUserPass().setString( settingsRecord.get( "password" ) );
            getLocale().setString( localeSetting );

            // Main Screen setting
            this.currentMainScreen = Short.parseShort( settingsRecord.get( "mainscreen" ) );

        }
        catch ( RecordStoreException rse )
        {
        }
    }

    // Connect to server to download everything (dataset, program, etc . .)
    public void downloadAll()
    {
        ConnectionManager.downloadAllResources();
    }

    // Load Data Set from Record Store base on Data Set ID
    public DataSet fetchDataSet( int dataSetId )
    {
        ModelRecordStore modelRecordStore = null;
        DataSet ds = null;

        try
        {
            modelRecordStore = new ModelRecordStore( ModelRecordStore.DATASET_DB );
            byte rec[] = modelRecordStore.getRecord( dataSetId );
            if ( rec != null )
                ds = new DataSet();
            try
            {
                SerializationUtil.deSerialize( ds, rec );
            }
            catch ( IOException ex )
            {
                ex.printStackTrace();
            }
        }
        catch ( RecordStoreException rse )
        {
        }

        return ds;
    }

    public void saveDataSet( DataSet dataSet )
    {
        if ( dataSet == null )
        {
            return;
        }
        ModelRecordStore modelRecordStore;

        try
        {

            modelRecordStore = new ModelRecordStore( ModelRecordStore.DATASET_DB );
            try
            {
                modelRecordStore.add( dataSet );
            }
            catch ( IOException ex )
            {
                ex.printStackTrace();
            }

        }
        catch ( RecordStoreException rse )
        {
            rse.printStackTrace();
        }
    }

    public void saveProgram( Program program )
    {
        if ( program == null )
        {
            return;
        }

        try
        {
            ModelRecordStore programRecordStore = new ModelRecordStore( ModelRecordStore.PROGRAM_DB );
            programRecordStore.add( program );
        }
        catch ( IOException ex )
        {
            ex.printStackTrace();
        }
        catch ( RecordStoreException rse )
        {
            rse.printStackTrace();
        }
    }

    // Save Activity Plan to Record Store
    public void saveActivityPlan( ActivityPlan activityPlan )
        throws IOException
    {
        if ( activityPlan != null )
        {
            ActivityRecordStore activityRecordStore = new ActivityRecordStore();
            activityRecordStore.setActivityVector( activityPlan.getActivities() );
            activityRecordStore.save();
        }
    }

    /**
     * Display the uncompleted Data Set Entry
     */
    public void displayDataSetEntry( DataSet ds, DataSetValue dsValue )
    {
        selectedDataSet = ds;
        if ( DHISMIDlet.DEBUG )
        {
            System.out.println( "displaying dataset:" + selectedDataSet.getName() + "-" + selectedDataSet.getVersion() );
        }
        if ( ds == null )
        {
            loadDataSets( this.getDataSetDisplayList() );
            this.switchDisplayable( Alerts.getInfoAlert( Text.ERROR(), Text.FORM_UNAVAILABLE() ),
                getDataSetDisplayList() );
        }
        else
        {
            formVector = null;
            formVector = new Vector();
            System.gc();
            // Loop through all Section of the DataSet
            if ( dsValue == null )
            {
                displayBlankDataSetEntry( ds );
            }
            else
            {
                displayDataSetEntryWithValues( ds, dsValue );
            }
        }
        switchDisplayable( null, (Form) formVector.elementAt( currentSection ) );
    }

    private void displayDataSetEntryWithValues( DataSet ds, DataSetValue dsValue )
    {
        int sectionsSize = ds.getSections().size();
        StringBuffer buffer = null;
        for ( int i = 0; i < sectionsSize; i++ )
        {
            Section section = (Section) ds.getSections().elementAt( i );
            Vector des = section.getDataElements();
            Form dsForm = new Form( section.getName() );
            // Loop through all DataElement of each Section
            int dataElementsSize = des.size();
            for ( int j = 0; j < dataElementsSize; j++ )
            {
                DataElement de = (DataElement) des.elementAt( j );
                Vector categoryOptCombos = de.getCategoryOptionCombos().getModels();
                dsForm.append( de.getName() );
                // Loop through all categoryOptCombo of each DataElement
                int categoryOptCombosSize = categoryOptCombos.size();
                for ( int k = 0; k < categoryOptCombosSize; k++ )
                {
                    Model categoryOptCombo = (Model) categoryOptCombos.elementAt( k );
                    DataValue dv = null;
                    // Check condition to display CategoryOptCombo as the
                    // label
                    String label = null;
                    buffer = new StringBuffer();
                    buffer.append( de.getId() );
                    buffer.append( "-" );
                    buffer.append( categoryOptCombo.getId() );
                    if ( categoryOptCombosSize > 1 )
                    {
                        label = categoryOptCombo.getName();
                    }

                    // Identify DataElement Type to append suitable Item
                    if ( de.getType().equals( "date" ) )
                    {
                        DateField dateField = new DateField( label, DateField.DATE );
                        dsForm.append( dateField );
                        dataElements.put( buffer.toString(), dateField );

                        dv = (DataValue) dsValue.getDataValues().get( buffer.toString() );
                        if ( !dv.getVal().equals( "" ) && dv != null )
                        {
                            dateField.setDate( Period.stringToDate( dv.getVal() ) );
                        }
                    }
                    else if ( de.getType().equals( "bool" ) )
                    {
                        choiceGroup = new ChoiceGroup( label, Choice.POPUP );
                        choiceGroup.append( Text.PLEASE_SELECT(), null );
                        choiceGroup.append( Text.YES(), null );
                        choiceGroup.append( Text.NO(), null );
                        choiceGroup.setFitPolicy( Choice.TEXT_WRAP_ON );
                        dsForm.append( choiceGroup );
                        dataElements.put( buffer.toString(), choiceGroup );
                        dv = (DataValue) dsValue.getDataValues().get( buffer.toString() );
                        if ( dv.getVal().equals( "" ) && dv != null )
                        {
                            choiceGroup.setSelectedIndex( 0, true );
                        }
                        else if ( dv.getVal().equals( "true" ) && dv != null )
                        {
                            choiceGroup.setSelectedIndex( 1, true );
                        }
                        else
                        {
                            choiceGroup.setSelectedIndex( 2, true );
                        }

                    }
                    else if ( de.getType().equals( "int" ) )
                    {
                        TextField intField = new TextField( label, "", TEXT_NUMERIC, TextField.NUMERIC );
                        dsForm.append( intField );
                        dataElements.put( buffer.toString(), intField );
                        if ( dsValue.getDataValues() != null && dsValue.getDataValues().get( buffer.toString() ) != "" )
                        {
                            dv = (DataValue) dsValue.getDataValues().get( buffer.toString() );
                            if ( dv != null )
                                intField.setString( dv.getVal() );
                        }
                    }
                    else
                    {
                        TextField txtField = new TextField( label, "", TEXT_STRING, TextField.ANY );
                        dsForm.append( txtField );
                        dataElements.put( buffer.toString(), txtField );
                        if ( dsValue.getDataValues() != null && dsValue.getDataValues().get( buffer.toString() ) != "" )
                        {
                            dv = (DataValue) dsValue.getDataValues().get( buffer.toString() );
                            if ( dv != null )
                                txtField.setString( dv.getVal() );
                        }
                    }
                    categoryOptCombo = null;
                    buffer = null;
                    System.gc();

                }

                // Add Command for Form
                dsForm.setCommandListener( this );

                if ( i == 0 )
                {
                    if ( sectionsSize > 1 )
                    {
                        dsForm.addCommand( getDsFormNextCmd() );
                    }
                    dsForm.addCommand( getDsFormExitCmd() );
                }
                if ( i > 0 && i < sectionsSize - 1 )
                {
                    dsForm.addCommand( getDsFormNextCmd() );
                    dsForm.addCommand( getDsFormBackCmd() );
                }
                if ( i == sectionsSize - 1 )
                {
                    if ( sectionsSize > 1 )
                    {
                        dsForm.addCommand( getDsFormBackCmd() );
                    }
                    dsForm.addCommand( getDsFormCompleteCmd() );

                }

                de = null;
                dsForm.addCommand( getDsFormSaveCmd() );
            }
            formVector.addElement( dsForm );
            dsForm = null;
            section = null;
            System.gc();
        }
    }

    private void displayBlankDataSetEntry( DataSet ds )
    {
        int sectionsSize = ds.getSections().size();
        StringBuffer buffer = null;
        for ( int i = 0; i < sectionsSize; i++ )
        {
            Section section = (Section) ds.getSections().elementAt( i );
            Vector des = section.getDataElements();
            Form dsForm = new Form( section.getName() );
            // Loop through all DataElement of each Section
            int dataElementsSize = des.size();
            for ( int j = 0; j < dataElementsSize; j++ )
            {
                DataElement de = (DataElement) des.elementAt( j );
                Vector categoryOptCombos = de.getCategoryOptionCombos().getModels();
                dsForm.append( de.getName() );
                // Loop through all categoryOptCombo of each DataElement
                int categoryOptCombosSize = categoryOptCombos.size();
                for ( int k = 0; k < categoryOptCombosSize; k++ )
                {
                    Model categoryOptCombo = (Model) categoryOptCombos.elementAt( k );
                    // Check condition to display CategoryOptCombo as the
                    // label
                    String label = null;
                    buffer = new StringBuffer();
                    buffer.append( de.getId() );
                    buffer.append( "-" );
                    buffer.append( categoryOptCombo.getId() );
                    if ( categoryOptCombosSize > 1 )
                    {
                        label = categoryOptCombo.getName();
                    }

                    // Identify DataElement Type to append suitable Item
                    if ( de.getType().equals( "date" ) )
                    {
                        DateField dateField = new DateField( label, DateField.DATE );
                        dsForm.append( dateField );
                        dataElements.put( buffer.toString(), dateField );
                    }
                    else if ( de.getType().equals( "bool" ) )
                    {
                        choiceGroup = new ChoiceGroup( label, Choice.POPUP );
                        choiceGroup.append( Text.PLEASE_SELECT(), null );
                        choiceGroup.append( Text.YES(), null );
                        choiceGroup.append( Text.NO(), null );
                        choiceGroup.setFitPolicy( Choice.TEXT_WRAP_ON );
                        dsForm.append( choiceGroup );
                        dataElements.put( buffer.toString(), choiceGroup );

                    }
                    else if ( de.getType().equals( "int" ) )
                    {
                        TextField intField = new TextField( label, "", TEXT_NUMERIC, TextField.NUMERIC );
                        dsForm.append( intField );
                        dataElements.put( buffer.toString(), intField );
                    }
                    else
                    {
                        TextField txtField = new TextField( label, "", TEXT_STRING, TextField.ANY );
                        dsForm.append( txtField );
                        dataElements.put( buffer.toString(), txtField );
                    }
                    categoryOptCombo = null;
                    buffer = null;
                    System.gc();

                }
                de = null;
            }
            if ( !(dataElementsSize > 0) )
            {
                dsForm.append( Text.EMPTY_DATASET() );

            }
            // Add Command for Form
            dsForm.setCommandListener( this );

            if ( dataElementsSize > 0 )
            {
                if ( i == 0 )
                {
                    if ( sectionsSize > 1 )
                    {
                        dsForm.addCommand( getDsFormNextCmd() );
                    }
                }
                if ( i > 0 && i < sectionsSize - 1 )
                {
                    dsForm.addCommand( getDsFormNextCmd() );
                    dsForm.addCommand( getDsFormBackCmd() );
                }
                if ( i == sectionsSize - 1 )
                {
                    if ( sectionsSize > 1 )
                    {
                        dsForm.addCommand( getDsFormBackCmd() );
                        dsForm.addCommand( getDsFormExitCmd() );
                    }
                    dsForm.addCommand( getDsFormCompleteCmd() );

                }
                dsForm.addCommand( getDsFormSaveCmd() );
            }
            if ( i == 0 )
                dsForm.addCommand( getDsFormExitCmd() );
            // add form to Vector of Form (List of Sections)
            formVector.addElement( dsForm );
            dsForm = null;
            section = null;
            System.gc();
        }
    }

    // Supportive method to collect Data Set Value from the Form
    public DataSetValue generateDataSetValue()
    {
        StringBuffer buffer = null;
        Vector des = getAllDataElement( selectedDataSet );
        DataSetValue dsValue = new DataSetValue();
        dsValue.setId( selectedDataSet.getId() );
        dsValue.setName( selectedDataSet.getName() );
        dsValue.setpName( selectedPeriod );

        String val;
        for ( int i = 0; i < des.size(); i++ )
        {
            DataElement de = (DataElement) des.elementAt( i );
            Vector categoryOptCombos = de.getCategoryOptionCombos().getModels();
            // Loop through all CategoryOptionCombo of the DataElement
            for ( int j = 0; j < categoryOptCombos.size(); j++ )
            {
                DataValue dv = new DataValue();
                Model categoryOptCombo = (Model) categoryOptCombos.elementAt( j );
                buffer = new StringBuffer();
                buffer.append( de.getId() );
                buffer.append( "-" );
                buffer.append( categoryOptCombo.getId() );
                // Identify DataElement Type
                if ( de.getType().equals( "date" ) )
                {
                    DateField dateField = (DateField) dataElements.get( buffer.toString() );
                    if ( dateField.getDate() != null )
                    {
                        val = Period.formatDate( dateField.getDate() );
                    }
                    else
                    {
                        val = "";
                    }
                }
                else if ( de.getType().equals( "int" ) )
                {
                    TextField intField = (TextField) dataElements.get( buffer.toString() );
                    val = intField.getString();
                    while ( val.indexOf( '0' ) == 0 )
                    {
                        val = val.substring( 1, val.length() );
                    }
                }
                else if ( de.getType().equals( "bool" ) )
                {
                    ChoiceGroup boolField = (ChoiceGroup) dataElements.get( buffer.toString() );

                    if ( boolField.isSelected( 0 ) )
                    {
                        val = "";
                    }
                    else if ( boolField.isSelected( 1 ) )
                    {
                        val = "true";
                    }
                    else
                    {
                        val = "false";
                    }
                }
                else
                {
                    TextField txtField = (TextField) dataElements.get( buffer.toString() );
                    val = txtField.getString();
                }
                dv.setId( de.getId() );
                dv.setCategoryOptComboID( categoryOptCombo.getId() );
                dv.setVal( val );
                dsValue.getDataValues().put( buffer.toString(), dv );
                dv = null;
                buffer = null;
                System.gc();

            }
            de = null;
        }
        System.gc();
        return dsValue;
    }

    // Send Data Set Value to server
    public void sendDataSetValue()
    {
        this.getWaitForm().deleteAll();
        this.getWaitForm().setTitle( Text.UPLOADING() );
        this.getWaitForm().append( Text.PLEASE_WAIT() );
        this.switchDisplayable( null, getWaitForm() );

        final DataSetValue dsValue = generateDataSetValue();
        loadSettings();
        this.saveDataSetValue( dsValue );

        int counter = emptyValues( dsValue );

        if ( counter == 0 )
        {
            formVector = null;
            ConnectionManager.uploadFacilityReport( dsValue );
        }
        else
        {
            CommandListener listener = new CommandListener()
            {

                public void commandAction( Command c, Displayable displayable )
                {
                    if ( c.getCommandType() == Command.OK )
                    {
                        DHISMIDlet.this.switchDisplayable( null, getWaitForm() );
                        ConnectionManager.uploadFacilityReport( dsValue );
                    }
                    else if ( c.getCommandType() == Command.CANCEL )
                    {
                        switchDisplayable( null, (Form) formVector.elementAt( currentSection ) );
                    }
                }
            };
            Alert alert = new Alert( Text.WARNING(), counter + " " + Text.FIELD_NOT_FILLED(), null,
                AlertType.CONFIRMATION );
            alert.addCommand( new Command( Text.YES(), Command.OK, 0 ) );
            alert.addCommand( new Command( Text.NO(), Command.CANCEL, 0 ) );
            alert.setCommandListener( listener );
            switchDisplayable( alert, getWaitForm() );
        }

        if ( selectedDataSet.getPeriodType().equals( "Monthly" ) )
        {
            DataSetValueRecordStore.clearOldDataSetValue( selectedDataSet.getId(), Period.MONTHLY,
                Period.generateMonthlyPeriods() );
        }
        else if ( selectedDataSet.getPeriodType().equals( "Yearly" ) )
        {
            DataSetValueRecordStore.clearOldDataSetValue( selectedDataSet.getId(), Period.YEARLY,
                Period.generateYearlyPeriods() );
        }
        else if ( selectedDataSet.getPeriodType().equals( "Weekly" ) )
        {
            DataSetValueRecordStore.clearOldDataSetValue( selectedDataSet.getId(), Period.WEEKLY,
                Period.generateWeeklyPeriods() );
        }
        else if ( selectedDataSet.getPeriodType().equals( "Quarterly" ) )
        {
            DataSetValueRecordStore.clearOldDataSetValue( selectedDataSet.getId(), Period.QUARTERLY,
                Period.generateQuaterlyPeriods() );
        }
        else if ( selectedDataSet.getPeriodType().equals( "Daily" ) )
        {
            DataSetValueRecordStore.clearOldDataSetValue( selectedDataSet.getId(), Period.DAILY, null );
        }

        System.gc();

    }

    // Supportive method to detect empty value
    private int emptyValues( DataSetValue dsValue )
    {
        int counter = 0;
        Enumeration enumeration = dsValue.getDataValues().elements();
        while ( enumeration.hasMoreElements() )
        {
            DataValue value = (DataValue) enumeration.nextElement();
            if ( value.getVal().equals( "" ) || value.getVal() == null )
            {
                counter++;
            }
        }

        return counter;
    }

    // Save updated DataSets to Record Store
    public void saveUpdatedDataSet( DataSetList dataSetList )
        throws IOException
    {
        Vector dataSets = null;
        ModelRecordStore modelRs = new ModelRecordStore( ModelRecordStore.DATASET_DB );
        try
        {
            if ( (dataSets = dataSetList.getAddedDataSets()) != null )
            {
                for ( int i = 0; i < dataSets.size(); i++ )
                {
                    modelRs.add( (DataSet) dataSets.elementAt( i ) );
                }
            }
            if ( (dataSets = dataSetList.getDeletedDataSets()) != null )
            {
                for ( int i = 0; i < dataSets.size(); i++ )
                {
                    modelRs.deleteRecord( (DataSet) dataSets.elementAt( i ) );
                }
            }
            // handle modified datasets
            if ( (dataSets = dataSetList.getModifiedDataSets()) != null )
            {
                for ( int i = 0; i < dataSets.size(); i++ )
                {
                    modelRs.add( (DataSet) dataSets.elementAt( i ) );
                }
            }
        }
        catch ( RecordStoreException e )
        {
            switchDisplayable( Alerts.getErrorAlert( Text.ERROR(), Text.ERROR_RMS() ), getDataSetDisplayList() );
            e.printStackTrace();
        }
    }

    // Save Data Set Value to Record Store
    public String saveDataSetValue( DataSetValue dsValue )
    {
        return DataSetValueRecordStore.saveDataSetValue( dsValue );
    }

    // Send Activity Value to server
    public void sendActivityValue()
    {
        this.saveActivityValue();
        this.getWaitForm().deleteAll();
        this.getWaitForm().setTitle( Text.UPLOADING() );
        this.getWaitForm().append( Text.PLEASE_WAIT() );
        this.switchDisplayable( null, this.getWaitForm() );

        final ActivityValue activityValue = generateActivityValue();
        int numOfEmptyValues = emptyValues( activityValue );
        if ( numOfEmptyValues == 0 )
        {
            loadSettings();
            ConnectionManager.uploadActivityReport( activityValue );
        }
        else
        {
            CommandListener listener = new CommandListener()
            {

                public void commandAction( Command c, Displayable d )
                {
                    if ( c.getCommandType() == Command.OK )
                    {
                        DHISMIDlet.this.switchDisplayable( null, getWaitForm() );
                        loadSettings();
                        ConnectionManager.uploadActivityReport( activityValue );
                    }
                    else if ( c.getCommandType() == Command.CANCEL )
                    {
                        switchDisplayable( null, getActivityEntryForm() );
                    }
                }
            };

            Alert alert = new Alert( Text.WARNING(), numOfEmptyValues + " " + Text.FIELD_NOT_FILLED(), null,
                AlertType.CONFIRMATION );
            alert.addCommand( new Command( Text.YES(), Command.OK, 0 ) );
            alert.addCommand( new Command( Text.NO(), Command.CANCEL, 0 ) );
            alert.setCommandListener( listener );
            switchDisplayable( alert, getWaitForm() );
        }
    }

    // Change Activity Status to Completed
    public void updateActivityStatus()
    {
        selectedActivity.getTask().setComplete( true );
        ActivityRecordStore activityRecordStore = new ActivityRecordStore( this );
        try
        {
            activityRecordStore.updateActivityStatus( selectedActivity );
        }
        catch ( IOException e )
        {
            System.out.println( "Couldn't update activity " + selectedActivity.getBeneficiary().getFullName() );
            e.printStackTrace();
        }
        activityRecordStore = null;
        System.gc();
    }

    // Supportive method to detect empty value
    private int emptyValues( ActivityValue activityValue )
    {
        int counter = 0;
        for ( int i = 0; i < selectedProgramStage.getDataElements().size(); i++ )
        {
            DataElement de = (DataElement) selectedProgramStage.getDataElements().elementAt( i );
            DataValue value = (DataValue) activityValue.getDataValues().get( String.valueOf( de.getId() ) );

            if ( (value == null || value.getVal().equals( "" )) && de.isCompulsory() )
            {
                counter++;
            }

            de = null;
            value = null;
            System.gc();
        }
        return counter;
    }

    // Save Activity Value to Record Store
    private String saveActivityValue()
    {
        ActivityValueRecordStore activityValueRecordStore = new ActivityValueRecordStore();
        return activityValueRecordStore.saveActivityValue( generateActivityValue() );
    }

    // Supportive method to collect Activity Value

    private ActivityValue generateActivityValue()

    {
        Vector des = selectedProgramStage.getDataElements();
        ActivityValue activityValue = new ActivityValue();
        activityValue.setProgramInstanceId( selectedActivity.getTask().getProgStageInstId() );
        DataElement de;
        String val = null;

        for ( int i = 0; i < des.size(); i++ )
        {
            de = (DataElement) des.elementAt( i );
            Vector categoryOptCombos = de.getCategoryOptionCombos().getModels();
            DataValue dv = new DataValue();
            dv.setCategoryOptComboID( ((Model) categoryOptCombos.elementAt( 0 )).getId() );
            if ( de.getType().equals( "date" ) )
            {
                if(serverCurrentDate != null){
                    ChoiceGroup dateCbx = (ChoiceGroup) dataElements.get( String.valueOf( de.getId() ) );
                    if ( dateCbx.isSelected( 0 ) ){
                        val = Period.formatDate( serverCurrentDate );
                    }else{
                        DateField dateField = (DateField) dateFieldValues.get( String.valueOf( de.getId() ) );
                        if( dateField != null ){
                            val = Period.formatDate( dateField.getDate() );
                        }else{
                            val = "";
                        }
                    }
                }else{
                    DateField dateField = (DateField) dataElements.get( String.valueOf( de.getId() ) );
                    if ( dateField.getDate() != null )
                    {
                        val = Period.formatDate( dateField.getDate() );
                    }
                    else
                    {
                        val = "";
                    }
                }
                
            }
            else if ( de.getType().equals( "int" ) )
            {
                TextField intField = (TextField) dataElements.get( String.valueOf( de.getId() ) );
                val = intField.getString();
            }
            else if ( de.getType().equals( "bool" ) )
            {
                ChoiceGroup boolField = (ChoiceGroup) dataElements.get( String.valueOf( de.getId() ) );
                if ( boolField.isSelected( 0 ) )
                {
                    val = "";
                }
                else if ( boolField.isSelected( 1 ) )
                {
                    val = "true";
                }
                else
                {
                    val = "false";
                }
            }
            else if ( de.getType().equals( "string" ) )
            {
                if ( de.getCategoryOptionCombos().getModels().size() <= 1 )
                {
                    TextField txtField = (TextField) dataElements.get( String.valueOf( de.getId() ) );
                    val = txtField.getString();
                }
                else
                {
                    ChoiceGroup choiceGroup = (ChoiceGroup) dataElements.get( String.valueOf( de.getId() ) );
                    int selectedIndex = choiceGroup.getSelectedIndex();
                    if ( selectedIndex == 0 )
                    {
                        val = "";
                    }
                    else
                    {
                        Model selectedCateOptCombo = (Model) categoryOptCombos.elementAt( selectedIndex - 1 );
                        dv.setCategoryOptComboID( selectedCateOptCombo.getId() );
                        val = String.valueOf( selectedCateOptCombo.getId() );
                    }
                }
            }
            dv.setId( de.getId() );
            dv.setVal( val );

            // if ( !val.trim().equalsIgnoreCase( "" ) )
            // {
            activityValue.getDataValues().put( String.valueOf( de.getId() ), dv );
            // }
        }
        return activityValue;

    }

    /**
     * Display main screen based on what data is available.
     */
    public void showMainScreen()
    {
        loadSettings();
        if ( currentMainScreen == GENERAL_MAINSCREEN )
        {
            this.getMainMenuList().addCommand( getMnuListBackCmd() );
            this.getDataSetDisplayList().addCommand( getDsLstBakCmd() );
            this.switchDisplayable( null, this.getServicesList() );
        }
        else if ( currentMainScreen == DATASET_MAINSCREEN )
        {
            this.getDataSetDisplayList().addCommand( getDsLstExitCmd() );
            this.loadDataSets( getDataSetDisplayList() );
            this.switchDisplayable( null, this.getDataSetDisplayList() );
        }
        else if ( currentMainScreen == ACTIVITY_MAINSCREEN )
        {
            this.getMainMenuList().addCommand( getMnuListExtCmd() );
            this.switchDisplayable( null, this.getMainMenuList() );
        }

    }

    /**
     * Collect all {@link DataElement data elements} for a {@link DataSet data
     * set}
     */
    public Vector getAllDataElement( DataSet dataSet )
    {
        Vector des = new Vector();
        for ( int i = 0; i < dataSet.getSections().size(); i++ )
        {
            Section section = (Section) dataSet.getSections().elementAt( i );
            for ( int j = 0; j < section.getDataElements().size(); j++ )
            {
                des.addElement( section.getDataElements().elementAt( j ) );
            }
            section = null;
        }
        System.gc();
        return des;
    }

    public DateField getDailyPeriodDateField()
    {
        if ( dailyPeriodDateField == null )
        {
            dailyPeriodDateField = new DateField( "Select Date:", DateField.DATE );
            dailyPeriodDateField.setDate( new java.util.Date( System.currentTimeMillis() ) );
        }
        return dailyPeriodDateField;
    }

    public Command getSelectDailyPeriodCmd()
    {
        if ( selectDailyPeriodCmd == null )
        {
            selectDailyPeriodCmd = new Command( "Select Date", Command.OK, 1 );
        }
        return selectDailyPeriodCmd;
    }

    public void setOrgUnit( OrgUnit orgUnit )
    {
        this.orgUnit = orgUnit;
    }

    public Activity getSelectedActivity()
    {
        return selectedActivity;
    }

    public void setSelectedActivity( Activity selectedActivity )
    {
        this.selectedActivity = selectedActivity;
    }

    private Form getUneditableForm()
    {
        if ( this.uneditableForm == null )
        {
            uneditableForm = new Form( "" );
            // uneditableForm.addCommand(getUnEditableBckCmd());
            uneditableForm.setCommandListener( this );
        }
        return uneditableForm;
    }

    public void setUneditableForm( Form uneditableForm )
    {
        this.uneditableForm = uneditableForm;
    }

    private Command getUnEditableBckCmd()
    {
        if ( uneditableBckCmd == null )
        {
            uneditableBckCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return uneditableBckCmd;
    }

    public Command getDataSetUneditableBckCmd()
    {
        if ( dataSetUneditableBckCmd == null )
        {
            dataSetUneditableBckCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return dataSetUneditableBckCmd;
    }

    public void setUneditableBckCmd( Command uneditableBckCmd )
    {
        this.uneditableBckCmd = uneditableBckCmd;
    }

    public Command getMnuListBackCmd()
    {
        if ( mnuListBackCmd == null )
        {
            mnuListBackCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return mnuListBackCmd;
    }

    public void setMnuListBackCmd( Command mnuListBackCmd )
    {
        this.mnuListBackCmd = mnuListBackCmd;
    }

    public Command getDsLstExitCmd()
    {
        if ( dsLstExitCmd == null )
        {
            dsLstExitCmd = new Command( Text.EXIT(), Command.EXIT, 0 );
        }
        return dsLstExitCmd;
    }

    public void setDsLstExitCmd( Command dsLstExitCmd )
    {
        this.dsLstExitCmd = dsLstExitCmd;
    }

    public Command getDsFormNextCmd()
    {
        if ( dsFormNextCmd == null )
        {
            dsFormNextCmd = new Command( Text.NEXT(), Command.OK, 0 );
        }
        return dsFormNextCmd;
    }

    public void setDsFormNextCmd( Command dsFormNextCmd )
    {
        this.dsFormNextCmd = dsFormNextCmd;
    }

    public Command getDsFormBackCmd()
    {
        if ( dsFormBackCmd == null )
        {
            dsFormBackCmd = new Command( Text.BACK(), Command.BACK, 0 );
        }
        return dsFormBackCmd;
    }

    public void setDsFormBackCmd( Command dsFormBackCmd )
    {
        this.dsFormBackCmd = dsFormBackCmd;
    }

    public Command getDsFormCompleteCmd()
    {
        if ( dsFormCompleteCmd == null )
        {
            dsFormCompleteCmd = new Command( Text.COMPLETE(), Command.OK, 0 );
        }
        return dsFormCompleteCmd;
    }

    public void setDsFormCompleteCmd( Command dsFormCompleteCmd )
    {
        this.dsFormCompleteCmd = dsFormCompleteCmd;
    }

    public Command getDsFormExitCmd()
    {
        if ( dsFormExitCmd == null )
        {
            dsFormExitCmd = new Command( Text.CLOSE(), Command.BACK, 0 );
        }
        return dsFormExitCmd;
    }

    public void setDsFormExitCmd( Command dsFormExitCmd )
    {
        this.dsFormExitCmd = dsFormExitCmd;
    }

    public Command getDsFormSaveCmd()
    {
        if ( dsFormSaveCmd == null )
        {
            dsFormSaveCmd = new Command( Text.SAVE(), Command.SCREEN, 0 );
        }
        return dsFormSaveCmd;
    }

    public void setDsFormSaveCmd( Command dsFormSaveCmd )
    {
        this.dsFormSaveCmd = dsFormSaveCmd;
    }

    public int getCurrentMainScreen()
    {
        return currentMainScreen;
    }

    public void setCurrentMainScreen( int currentMainScreen )
    {
        this.currentMainScreen = currentMainScreen;
    }

    public Vector getOrgUnits()
    {
        return orgUnits;
    }

    public void setOrgUnits( Vector orgUnits )
    {
        this.orgUnits = orgUnits;
    }

    public Date getServerCurrentDate()
    {
        return serverCurrentDate;
    }

    public void setServerCurrentDate( Date serverCurrentDate )
    {
        this.serverCurrentDate = serverCurrentDate;
    }

    public List getOrgUnitSelectList()
    {
        if ( this.orgUnitSelectList == null )
        {
            this.orgUnitSelectList = new List( Text.SELECT_ORG_UNIT(), List.IMPLICIT );
            this.orgUnitSelectList.setFitPolicy( List.TEXT_WRAP_ON );
            this.orgUnitSelectList.setCommandListener( this );
        }
        return this.orgUnitSelectList;
    }

    public void setOrgUnitSelectList( List orgUnitSelectList )
    {
        this.orgUnitSelectList = orgUnitSelectList;
    }

    public void saveInitSetting()
    {
        SettingsRecordStore settingsRecord = null;
        try
        {
            settingsRecord = new SettingsRecordStore( SettingsRecordStore.SETTINGS_DB );
            settingsRecord.put( "url", getUrl().getString() );
            settingsRecord.put( "username", getUserName().getString() );
            settingsRecord.put( "password", getPassword().getString() );
            settingsRecord.put( "pin", getPinTextField().getString().trim() );
            settingsRecord.put( "mainscreen", "" + currentMainScreen );
            settingsRecord.save();
        }
        catch ( RecordStoreException e )
        {
            e.printStackTrace();
        }
    }

    public void saveFacilityForms( Vector dataSets )
    {
        for ( int i = 0; i < dataSets.size(); i++ )
        {
            saveDataSet( (DataSet) dataSets.elementAt( i ) );
        }

        if ( this.isWaiting() )
        {
            this.saveInitSetting();
            showMainScreen();
        }

    }

    public void selectOrgUnit()
    {
        for ( int i = 0; i < this.getOrgUnits().size(); i++ )
        {
            OrgUnit orgUnit = (OrgUnit) this.getOrgUnits().elementAt( i );
            this.getOrgUnitSelectList().append( orgUnit.getName(), null );
        }

        this.switchDisplayable( null, this.getOrgUnitSelectList() );
    }

}
