package org.hisp.dhis.mobile.ui;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import org.hisp.dhis.mobile.model.TextMapping;

public class TextMappingLoader
{

    private static final String PREFIX = "/properties/";

    private static final String SUFFIX = ".properties";

    public static TextMapping load( String locale )
        throws IOException
    {
        Class clazz = Runtime.getRuntime().getClass();
        if ( locale.equals( "en" ) )
        {
            locale = locale + "-US";
        }
        else if ( locale.equals( "vi" ) )
        {
            locale = locale + "-VN";
        }
        else if ( locale.equals( "hi" ) )
        {
            locale = locale + "-HI";
        }
        else
        {
            locale = "en-US";
        }
        String resourcePath = PREFIX + locale + SUFFIX;
        InputStream is = clazz.getResourceAsStream( resourcePath );
        if ( is == null )
        {
            // FIXME
            throw new IOException( Text.RESOURCE_FILE() + " " + resourcePath + " " + Text.DOES_NOT_EXIST() );
        }

        Hashtable textTable = parsePropertiesFile( is );
        TextMapping textMapping = new TextMapping();
        textMapping.setTextTable( textTable );
        return textMapping;
    }

    public static Hashtable parsePropertiesFile( InputStream is )
        throws IOException
    {
        Hashtable textTable = new Hashtable();
        int charRead = 0;
        StringBuffer messageKey = new StringBuffer();
        StringBuffer messageText = new StringBuffer();
        boolean readingKey = true;

        while ( (charRead = is.read()) != -1 )
        {
            if ( charRead == '=' )
            {
                readingKey = false;
            }
            else if ( charRead == '\r' )
            {
                is.read();
                if ( messageKey.length() > 0 )
                {
                    textTable.put( messageKey.toString(), convertString( messageText.toString() ) );
                    messageKey = new StringBuffer();
                    messageText = new StringBuffer();
                    readingKey = true;
                }
            }
            else if ( charRead == '#' )
            {
                while ( is.read() != '\n' )
                {
                }
            }
            else
            {
                if ( readingKey )
                    messageKey.append( (char) charRead );
                else
                    messageText.append( (char) charRead );
            }
        }
        return textTable;
    }

    private static String convertString( String theString )
    {
        char aChar;
        int len = theString.length();
        StringBuffer outBuffer = new StringBuffer( len );

        for ( int x = 0; x < len; )
        {
            aChar = theString.charAt( x++ );
            if ( aChar == '\\' )
            {
                aChar = theString.charAt( x++ );
                if ( aChar == 'u' )
                {
                    int value = 0;
                    for ( int i = 0; i < 4; i++ )
                    {
                        aChar = theString.charAt( x++ );
                        switch ( aChar )
                        {
                        case '0':
                        case '1':
                        case '2':
                        case '3':
                        case '4':
                        case '5':
                        case '6':
                        case '7':
                        case '8':
                        case '9':
                            value = (value << 4) + aChar - '0';
                            break;
                        case 'a':
                        case 'b':
                        case 'c':
                        case 'd':
                        case 'e':
                        case 'f':
                            value = (value << 4) + 10 + aChar - 'a';
                            break;
                        case 'A':
                        case 'B':
                        case 'C':
                        case 'D':
                        case 'E':
                        case 'F':
                            value = (value << 4) + 10 + aChar - 'A';
                            break;
                        default:
                            return "NO_STRING";
                        }
                    }
                    outBuffer.append( (char) value );
                }
                else
                {
                    if ( aChar == 't' )
                        aChar = '\t';
                    else if ( aChar == 'r' )
                        aChar = '\r';
                    else if ( aChar == 'n' )
                        aChar = '\n';
                    else if ( aChar == 'f' )
                        aChar = '\f';
                    outBuffer.append( aChar );
                }
            }
            else
            {
                outBuffer.append( aChar );
            }
        }
        return outBuffer.toString();
    }

}
